#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, math, time, array, glob

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt', 'TheSecondWorldHeaven']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

sys.path.append('/usr/local/IcePy-3.2.0/python')
import Ice, IceStorm
import Image, ImageDraw
Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I../../../slice -I/usr/share/slice Magarro.ice')
import FIPA
sys.path.append('../../python')
import AgentI
from AgentI import *
import Zone, Util, Util_image
from Zone import *

# Minimum width and height of the smallest work.
FACTOR_X_MIN, FACTOR_Y_MIN = 6, 4
# Division factors of the zones in the final adjustment.
FACTOR_DIV1, FACTOR_DIV2 = 1.5, 3
# Standard desviation allowed for the work colour.
DESV = 30.0
# Line colours.
LEVEL1 = 255
LEVEL2 = 175
LEVEL3 = 100
# Maximum difference of the colour media in a zone.
MAX_COLOUR = 20.0

if HOST not in LOCAL_HOSTS:
	MASYRO_WEB_DIR = MAGARRO_HOME + '/gui'
	ANALYST_WORK_DIR = MAGARRO_HOME + '/analyst'
	RENDER_WORK_DIR = MAGARRO_HOME + '/render'
	NAME_OUTPUT_FILE = 'output.png'
else:
	MASYRO_WEB_DIR = '/var/www/magarro'
	FROM_RENDER_DIR = '../magarro/analyst'
	ANALYST_WORK_DIR = './analyst'
	RENDER_WORK_DIR = './render'
	NAME_OUTPUT_FILE = 'output.png'

class AnalystI (FIPA.Analyst, AgentI):
    
    def __init__ (self, name):
        """Crea un objeto del tipo Analyst"""
        
        AgentI.__init__(self, None)
        self.Name = name
        
        # InputFile --> Object of image type that represents the final image.
        self.InputFile = None
        # Resolution of InputFile.
        self.X, self.Y = 0, 0
        # Level == 1 --> Standard division Level == 2 --> Division with fusion Level == 3 --> Complexity balance.
        self.Level = 0
        # PixelMatrix represents the matrix of pixels of InputFile.
        self.PixelMatrix = []
        # Zones of the image depending on the level parameter.
        self.InitialZones, self.MergingZones, self.FinalZones = [], [], []
	# File info parameters (in string format, from infoscene.txt)
	self.InfoScene = ""
        
        # Proxies.
        self.Master, self.Repository, self.Blackboard = None, None, None

    def getFileInfo(self, curr):
	return self.InfoScene
	
    def getZonesByPartition (self, partition, curr):

        zones= []
        if (partition == 1):
            for x in self.InitialZones:
                zones.append(FIPA.TZone(x.Id, x.X1, x.Y1, x.X2, x.Y2, x.Desv, x.Med))
        if (partition == 2):
            for x in self.MergingZones:
                zones.append(FIPA.TZone(x.Id, x.X1, x.Y1, x.X2, x.Y2, x.Desv, x.Med))
	if (partition == 3):
            for x in self.FinalZones:
                zones.append(FIPA.TZone(x.Id, x.X1, x.Y1, x.X2, x.Y2, x.Desv, x.Med))

        return zones

    def initialRender (self, file):

        unzip = Util.Unzip(file)
        # Store the file names.
        nameList = unzip.extract(ANALYST_WORK_DIR)
        # Obtain the full name of the file which contains the scene.
        scene = Util.getFileName(nameList, 'blend')
        path = os.path.join(ANALYST_WORK_DIR, scene)
        # Estimated render.
        os.system('blender -b ' + path + ' -P ./InitialRender.py -f 1 ')
        path1 = os.path.join(ANALYST_WORK_DIR, '0001.png')
        path2 = os.path.join(ANALYST_WORK_DIR, NAME_OUTPUT_FILE)
        os.system('mv ' + path1 + ' ' + path2)
	# Get File Info
	os.system('blender -b ' + path + ' -P ./FileInfo.py')
	f = open(os.path.join(ANALYST_WORK_DIR, 'infoscene.txt'), 'r+')
	self.InfoScene = f.readline()
	self.InfoScene = self.InfoScene + ":" + scene + ":" + str(os.stat(path).st_size) 
	self.InfoScene = self.InfoScene + ":" + str(os.stat(path).st_mtime)
	f.close()
	print "Analyst: File info: " + self.InfoScene
	Util_image.createLowResolutionImage(ANALYST_WORK_DIR, NAME_OUTPUT_FILE, MASYRO_WEB_DIR)

    def getInfoInitialImage (self, inputFile):

        self.InputFile = Image.open(inputFile)
        # Obtain the image resolution.
        x, y = self.InputFile.size
        self.X = x
        self.Y = y

        for i in range (self.X):
            aux = []
            for j in range (self.Y):
                aux.append(self.InputFile.getpixel((i, j)))
            self.addZone(aux, self.PixelMatrix)

    def getSmallestPieceX (self):
        return self.X / FACTOR_X_MIN

    def getSmallestPieceY (self):
        return self.X / FACTOR_Y_MIN

    def setPixelMatrix (self, i, j, p):
        self.PixelMatrix[i][j] = p

    def getPixelMatrixValue (self, i, j):
        return self.PixelMatrix[i][j]

    def addZone (self, zone, zoneList):
        zoneList.append(zone)

    def deleteZone (self, zone, zoneList):
        zoneList.remove(zone)

    def getCurrentAnalysisTime (self, curr):
        return self.getAnalysisTime()    

    def getId (self):

        if len(self.InitialZones) == 0:
            return 0
        else:
            return self.InitialZones[len(self.InitialZones) - 1].Id + 1

    def getIdAjuste (self):
        return self.FinalZones[len(self.FinalZones) - 1].Id + 1

    def divideZone (self, zone):

        x1, y1 = zone.X1, zone.Y1
        x2, y2 = zone.X2, zone.Y2

        # Study if the zone can be divided.
        med, d = self.getMediumStandardDesviation(zone)

        # Division if the work is big and the standard desviation is bigger than the established.
        if (d > DESV) and zone.getWidth() > self.getSmallestPieceX() and zone.getHeight() > self.getSmallestPieceY():
            self.divideZone(Zone(-1, x1, y1, x1 + (x2 - x1) / 2, y1 + (y2 - y1) / 2))
            self.divideZone(Zone(-1, x1 + (x2 - x1) / 2, y1, x2, y1 + (y2 - y1) / 2))
            self.divideZone(Zone(-1, x1, y1 + (y2 - y1) / 2, x1 + (x2 - x1) / 2, y2))
            self.divideZone(Zone(-1, x1 + (x2 - x1) / 2, y1 + (y2 - y1) / 2, x2, y2))

        # Not divisible zone.
        else:
            zone = Zone(self.getId(), x1, y1, x2, y2)
            zone.Desv = d
            zone.Med = med
            self.addZone(zone, self.InitialZones)            

    def dibujarLineasSeparacion (self, ficheroSalida, zonas, fill):

        draw = ImageDraw.Draw(self.InputFile)
        FILL = fill

        for x in zonas:
            draw.line((x.X1, x.Y1, x.X2, x.Y1), FILL)
            draw.line((x.X2, x.Y1, x.X2, x.Y2), FILL)
            draw.line((x.X1, x.Y1, x.X1, x.Y2), FILL)
            draw.line((x.X1, x.Y2, x.X2, x.Y2), FILL)

        self.InputFile.save(ficheroSalida)

        del draw

    def getMediumStandardDesviation (self, zone):

        sume, numData = 0, 0

        # Average calculation.
        for i in range (zone.X1, zone.X2):
            for j in range (zone.Y1, zone.Y2):
                sume += self.getPixelMatrixValue(i, j)
                numData += 1

        medium = float(sume) / float(numData)

        # Standard desviation calculation.
        sume = 0
        for i in range (zone.X1, zone.X2):
            for j in range (zone.Y1, zone.Y2):
                sume += math.pow((self.getPixelMatrixValue(i, j) - medium), 2)

        desv = math.sqrt(float(sume) / float(numData))

        return medium, desv        

    def fusion (self):

        change = True

        # Set the result zone list to the initial zones.
        for x in self.InitialZones:
            self.addZone(x, self.MergingZones)

        # Study the zone list if there are changes.
        while change:
            change = False
            for x in self.MergingZones:
                for y in self.MergingZones:
                    # If we have two zones with a colour average similar...
                    if math.fabs(x.Med - y.Med) < MAX_COLOUR:
                        # Insert the new zone if needed.
                        aux = self.adjacency(x, y)
                        if aux == True:
                            change = True

    def adjacency (self, x, y):

        # Vertical adjacency.
        if (x.X1 == y.X1) and (x.Y2 == y.Y1) and (x.X2 == y.X2):
            # Fusion changing the coordinate y2.
            x.Y2 = y.Y2
            self.deleteZone(y, self.MergingZones)

        # Horizontal adjacency.
        elif (x.X2 == y.X1) and (x.Y1 == y.Y1) and (x.Y2 == y.Y2):
            # Fusion changing the coordinate x2.
            x.X2 = y.X2
            medium, d = self.getMediumStandardDesviation(x)
            x.Med = medium
            x.Desv = d
            self.deleteZone(y, self.MergingZones)

        else:
            return False

        return True

    def balance (self):

        # Complexity of the zones (colour).
        complexity = []
        aux = []

        # Initialize with the fusion zones.
        for x in self.MergingZones:
            self.addZone(x, self.FinalZones)

        for x in self.FinalZones:
            c = 0
            for i in range(x.X1, x.X2 - 1):
                for j in range(x.Y1, x.Y2 - 1):
                    c += self.PixelMatrix[i][j]
            complexity.append(c / x.getSize())

        medium, desv = self.getStandardDesviationBalance(complexity)

        for i in range(len(complexity)):
            if (medium) <= complexity[i]:
                zona = self.FinalZones[i]
                self.divideFinalZone(zona, medium)
                aux.append(zona)

        for x in aux:
            self.deleteZone(x, self.FinalZones)
                
    def getStandardDesviationBalance (self, complexity):

        sume, numData = 0, 0

        # Average calculation.
        for x in complexity:
            sume += x
            numData += 1

        medium = float(sume) / float(numData)

        # Standard desviation calculation.
        sume = 0
        for x in complexity:
            sume += math.pow((x - medium), 2)

        desv = math.sqrt(float(sume) / float(numData))

        return medium, desv

    def divideFinalZone (self, zone, med):

        # Erase the complex zone depending on the complexity-size ratio.
        x1, y1 = zone.X1, zone.Y1
        x2, y2 = zone.X2, zone.Y2

        # Append a zone which is the half of the erased zone.
        z1 = Zone(self.getIdAjuste(), x1, y1, x2, y1 + (y2 - y1) / 2)
        medium, d = self.getMediumStandardDesviation(z1)
        z1.Med = medium
        z1.Desv = d
        self.addZone(z1, self.FinalZones)

        # Append a zone which is the other half of the erased zone.
        z2 = Zone(self.getIdAjuste(), x1, y1 + (y2 - y1) / 2, x2, y2)
        medium, d = self.getMediumStandardDesviation(z2)
        z2.Med = medium
        z2.Desv = d
        self.addZone(z2, self.FinalZones)

        # If the zone is bigger than the double of the average...
        if (zone.Med >= med * FACTOR_DIV1):

            x1, y1 = z1.X1, z1.Y1
            x2, y2 = z1.X2, z1.Y2
            # Append a zone which is the left vertical half of the erased zone.
            zvi = Zone(self.getIdAjuste(), x1, y1, x1 + (x2 - x1) / 2, y2)
            medium, d = self.getMediumStandardDesviation(zvi)
            zvi.Med = medium
            zvi.Desv = d
            self.addZone(zvi, self.FinalZones)
            # Append a zone which is the right vertical half of the erased zone.
            zvd = Zone(self.getIdAjuste(), x1 + (x2 - x1) / 2, y1, x2, y2)
            medium, d = self.getMediumStandardDesviation(zvd)
            zvd.Med = medium
            zvd.Desv = d
            self.addZone(zvd, self.FinalZones)
            # Erase the original zone.
            self.deleteZone(z1, self.FinalZones)

            x1, y1 = z2.X1, z2.Y1
            x2, y2 = z2.X2, z2.Y2
            # Append a zone which is the left vertical half of the erased zone.
            zvi = Zone(self.getIdAjuste(), x1, y1, x1 + (x2 - x1) / 2, y2)
            medium, d = self.getMediumStandardDesviation(zvi)
            zvi.Med = medium
            zvi.Desv = d
            self.addZone(zvi, self.FinalZones)
            # Append a zone which is the right vertical half of the erased zone.
            zvd = Zone(self.getIdAjuste(), x1 + (x2 - x1) / 2, y1, x2, y2)
            medium, d = self.getMediumStandardDesviation(zvd)
            zvd.Med = medium
            zvd.Desv = d
            self.addZone(zvd, self.FinalZones)
            # Erase the original zone.
            self.deleteZone(z2, self.FinalZones)

    def obtainMaster (self, comm):

        try:
            # Searching for a Master.
            self.Master = FIPA.MasterPrx.checkedCast(comm.stringToProxy('master'))
            return True

        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to a Master.'
            return False        

    def obtainBlackboard (self, comm):

        try:
            obj = comm.stringToProxy('blackboard')
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(queryPrx.findObjectByType('::FIPA::Blackboard'))
        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to the blackboard.'

    def obtainRepository (self, comm):

        try:
            obj = comm.stringToProxy('modelRepository')
            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(queryPrx.findObjectByType('::FIPA::ModelRepository'))
        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to the model repository.'        

    def notifyNewWorkToMaster (self, idWork, optimization, rendermethods, curr) :

        self.notifyNewWorkToMaster_Internal (idWork, optimization, rendermethods, curr.adapter.getCommunicator())

    def notifyNewWorkToMaster_Internal (self, idWork, optimization, rendermethods, comm):

        if self.obtainMaster(comm):
            print self.Name + ' --> Master obtained...'
            zones = []
            for x in self.getZones(self.Level):
		zones.append(FIPA.TZone(x.Id, x.X1, x.Y1, x.X2, x.Y2, x.Desv, x.Med))
            self.Master.notifyNewWork(zones, idWork, optimization, rendermethods)
            print self.Name + ' --> Notifying a new work to the master.'

        else:
            print self.Name + ' --> was not able to obtain a reference to the master.'

    def sendModelToRepository (self, zip, file, comm):

        try:
            self.obtainRepository(comm)
            return self.Repository.put(zip, file)

        except Ice.MemoryLimitException:
            print self.Name + ' --> Memory of the model repository exceeded'

    def mostrarPixelMatrix (self):

        for i in range (0, self.Y):
            aux = ''
            for j in range (0, self.X):                
                aux += self.getPixelMatrixValue(j, i).__str__() + ' '
            print aux

    def mostrarZones (self):
        
        for x in self.InitialZones:
            print x
        print 'Number of zones without merging: ', len(self.InitialZones)

    def mostrarMergingZones (self):
        
        for x in self.MergingZones:
            print x
        print 'Number of zones with merging: ', len(self.MergingZones)

    def mostrarFinalZones (self):
        
        for x in self.FinalZones:
            print x
        print 'Number of zones after adjustment: ', len(self.FinalZones)        

    def analyze (self):

        if self.Level >= 1:
            self.divideZone(Zone(-1, 0, 0, self.X, self.Y))
        if self.Level >= 2:
            self.fusion()
        if self.Level >= 3:
            self.balance()

    def draw (self):

        if self.Level == 1:
            self.dibujarLineasSeparacion(os.path.join(ANALYST_WORK_DIR, 'lineas.png'), self.InitialZones, LEVEL1)
        elif self.Level == 2:
            self.dibujarLineasSeparacion(os.path.join(ANALYST_WORK_DIR, 'lineasFusion.png'), self.MergingZones, LEVEL2)
        elif self.Level == 3:
            self.dibujarLineasSeparacion(os.path.join(ANALYST_WORK_DIR, 'lineasAjuste.png'), self.FinalZones, LEVEL3)

    def getZones (self, level):

        if self.Level == 1:
            return self.InitialZones
        elif self.Level == 2:
            return self.MergingZones
        elif self.Level == 3:
            return self.FinalZones

    def showZones (self):

        if self.Level == 1:
            self.mostrarZones()
        elif self.Level == 2:
            self.mostrarMergingZones()
        elif self.Level == 3:
            self.mostrarFinalZones()

    def registerAsWKO (self):

        properties = self.communicator().getProperties()
        adapter = self.communicator().createObjectAdapter('AnalystAdapter')
        id = Ice.stringToIdentity(properties.getProperty('IdentityAN'))

        self.init(properties.getProperty('InputAN'))

        adapter.add(self, id)
        adapter.activate()

    def processWork (self, work, workName, level, optimization, rendermethods, curr):

        self.obtainBlackboard(curr.adapter.getCommunicator())
        self.Blackboard.stSetUploadTime()

        begin = time.time()
        self.clear()

        print 'Processing ' + workName + '...'
        print 'Partition level ' + str(level) + '.'
        print 'Optimization level ' + str(optimization) + '.'
	print 'Render Methods ' + str(rendermethods) + '.\n'

        self.Level = level
	# Clean the state.
	self.PixelMatrix = []
	self.InitialZones, self.MergingZones, self.FinalZones = [], [], []
	self.InfoScene = ""
        # Store the received file.
        newWork = array.array('B', open(work).read()).tolist()
        path = os.path.join(ANALYST_WORK_DIR, workName)
        array.array('B', newWork).tofile(open(path, 'w'))

        # Estimated render.
        self.initialRender(path)
        self.getInfoInitialImage(os.path.join(ANALYST_WORK_DIR, NAME_OUTPUT_FILE))
        # Analyze the image obtained of the estimated render to obtain the work zones.
        self.analyze()
        # Create a image with the work zones.
        self.draw()
        self.showZones()

        end = time.time()
	self.Blackboard.stSetAnalysisTime()

        # Send the work to the repository and notify to the master.
        id = self.sendModelToRepository(workName, newWork, curr.adapter.getCommunicator())

	return id

    def clear (self):
        self.InitialZones, self.MergingZones, self.FinalZones = [], [], []

class Analyst(Ice.Application):

    def run(self, argv):
        self.shutdownOnInterrupt()
        
        adapter = self.communicator().createObjectAdapter("AgentAdapter")
        adapter.activate()
        
        id = self.communicator().stringToIdentity(argv[1])
        agentI = AnalystI(argv[1])
        proxy = adapter.add(agentI, id)
        agentPrx = FIPA.RenderAgentPrx.uncheckedCast(proxy)
        print 'Starting up the Analyst ' + agentPrx.ice_toString()

        prx = self.communicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession(argv[1], 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.addObjectWithType(agentPrx, agentPrx.ice_id())
        except IceGrid.ObjectExistsException:
            print agentPrx.ice_toString() + ' exists.'

        prx2 = self.communicator().stringToProxy('FIPA/Query')
        queryPrx = IceGrid.QueryPrx.checkedCast(prx2)
        obj2 = queryPrx.findObjectById(self.communicator().
                                       stringToIdentity('DF/topic.DirectoryFacilitator'))
        topic = IceStorm.TopicPrx.uncheckedCast(obj2)
        
        try:
            topic.subscribeAndGetPublisher(None, proxy)
        except IceStorm.AlreadySubscribed:
            print 'The analyst was subscribed to the topic.'
        except IceStorm.NoSuchTopic:
            print 'The topic DF/topic.DirectoryFacilitator does not exist.'

        obj4 = queryPrx.findObjectById(self.communicator().stringToIdentity('df'))
        dfPrx = FIPA.DirectoryFacilitatorPrx.uncheckedCast(obj4)
        if dfPrx:
            description = FIPA.DfAgentDescription()
            description.proxy = agentPrx
            agentDesc = FIPA.AgentDescription()
            agentDesc.leaseTime = 0
            s = []
            s.append(FIPA.ServiceDescription('Analyst', 'magarro'))            
            agentDesc.services = s
            description.desc = agentDesc
            dfPrx.register(description)

        obj3 = queryPrx.findObjectById(self.communicator().stringToIdentity('acc'))
        accPrx = FIPA.AgentCommunicationChannelPrx.uncheckedCast(obj3)
        print agentPrx.ice_toString() + ' is online.'

        self.communicator().waitForShutdown()

        topic.unsubscribe(proxy)
        agentI.clear()

        return 0

Analyst().main(sys.argv, '../../../config/config.agent')
