#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, array, time
sys.path.append('/usr/local/IcePy-3.2.0/python')
import Ice, IceGrid, IceStorm
###
## Ice.loadSlice('--all -I../../../slice -I. -I/usr/share/slice Magarro.ice')
###
Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I../../../slice -I/usr/share/slice Magarro.ice')
import FIPA
sys.path.append('../../python')
import AgentI
from AgentI import *
import Util

class BlackboardI (FIPA.Blackboard, AgentI):

    def __init__ (self, name):

        AgentI.__init__(self, None)
        self.Name = name
        # (Key, Value) == ((IdWork, IdZone), Register)
        self.Registers = {}
        # AnalysisTime represents the analysis time (seconds) of the most recent scene.
        self.AnalysisTime = 0
        # EstimatedRenderTime represents the time (seconds) of the work units estimated.
        self.EstimatedRenderTime = 0
        # Estimation Time: Time spent estimating the scene.
        self.EstimationTime = 0
        # Times for statistics
        self.stInitTime = 0
        self.stUploadTime = 0
        self.stAnalysisTime = 0
        self.stEstimationTime = 0
        self.stBiddingTime = 0
        self.stRenderTime = 0

    def EstimationTime(self, time, curr):
        self.EstimationTime = time

    def stGetCurrentTime (self, curr):
        return (int(time.time()))

    def stGetTimes(self, curr):
        return ((self.stInitTime, self.stUploadTime, self.stAnalysisTime, self.stEstimationTime, self.stBiddingTime, self.stRenderTime))

    def stResetTimes(self, curr):
        self.stInitTime = 0
        self.stUploadTime = 0
        self.stAnalysisTime = 0
        self.stEstimationTime = 0
        self.stBiddingTime = 0
        self.stRenderTime = 0

    def stSetInitTime(self, curr):
        self.stInitTime = int(time.time())
    def stSetUploadTime(self, curr):
        self.stUploadTime = int(time.time())
    def stSetAnalysisTime(self, curr):
        self.stAnalysisTime = int(time.time())
    def stSetEstimationTime(self, curr):
        self.stEstimationTime = int(time.time())
    def stSetBiddingTime(self, time, curr):
        self.stBiddingTime = int(time)
    def stSetRenderTime(self, curr):
        self.stRenderTime = int(time.time())

    def addRegister (self, register):
        self.Registers[(register.IdWork, register.WorkUnit)] = register

    def write (self, register, current = None):
        self.addRegister(register)

    def read (self, idWork, workUnit, current = None):

        if self.Registers.has_key((idWork, workUnit)):
            return self.Registers[(idWork, workUnit)]
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def update (self, idWork, workUnit, test, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].Test = test
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')
    def updateRT (self, idWork, workUnit, test, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].TestRT = test
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')
    def updateRC (self, idWork, workUnit, test, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].TestRC = test
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def updatePercRC (self, idWork, workUnit, perc, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].QDRC = perc
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def updatePercRT (self, idWork, workUnit, perc, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].QDRT = perc
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def updateMethod (self, idWork, workUnit, method, current = None):
        
        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].Method = method
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def setEstimationTime(self, time, curr):
        self.EstimationTime = time

    def getEstimationTime(self, curr):
        return self.EstimationTime

    def setAnalysisTime (self, time, curr):
        self.AnalysisTime = time
        
    def getAnalysisTime (self, curr):
        return self.AnalysisTime

    def incrementEstimatedRenderTime (self, time, curr):
        self.EstimatedRenderTime += time

    def getEstimatedRenderTime (self, curr):
        return self.EstimatedRenderTime

    def clear (self, curr):
        self.Registers.clear()
        self.EstimatedRenderTime = 0

    def isWorkPartiallyEstimated (self, curr):

        estimatedZones = 0

        for r in self.Registers.values():
            if r.Test <> 0:
                estimatedZones += 1

        if estimatedZones == len(self.Registers):
            return True
        else:
            return False

    def isCurrentWorkFinished (self, curr):

        for r in self.Registers.values():
            if r.State != FIPA.StateRegister.Done:
                return False

        return True

    def show (self, curr):
        return self.__str__()

    def getMaxTest (self, idWork, curr):

        maxTest = -1

        for x in self.Registers.values():
            if x.IdWork == idWork and x.Test > maxTest:
                maxTest = x.Test

        return maxTest

    def getMaxComp (self, idWork, curr):

        maxComp = -1

        for x in self.Registers.values():
            if x.IdWork == idWork and x.Comp > maxComp:
                maxComp = x.Comp

        return maxComp

    def setWorkUnit (self, idWork, workUnit, agent, curr):

        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].Agent = agent
            self.Registers[(idWork, workUnit)].State = FIPA.StateRegister.InWork
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def finishWorkUnit (self, idWork, workUnit, treal, ibs, ls, rl, curr):

        if self.Registers.has_key((idWork, workUnit)):
            self.Registers[(idWork, workUnit)].Treal = int(treal)
            self.Registers[(idWork, workUnit)].State = FIPA.StateRegister.Done
            self.Registers[(idWork, workUnit)].Ibs = ibs
            self.Registers[(idWork, workUnit)].Ls = ls
            self.Registers[(idWork, workUnit)].Rl = rl
        else:
            raise FIPA.RegisterNotExistsException('The register ' + str(idWork) + ', ' + str(workUnit) + ' does not exist.')

    def getTimeCurrentWorks (self, curr):

        agents = {}

        for x in self.Registers.values():
            if not x.Agent in agents.keys():
                agents[x.Agent] = x.Treal
            else:
                agents[x.Agent] += x.Treal

        seconds = max(agents.values())
        tTotal = '[H: ' + str(seconds / 3600) + ' M: ' + str(seconds % 3600 / 60) + ' S: ' + str(seconds % 3600 % 60) + ']\n'

        straux = 'Time spent: ' + tTotal
        
        for x in agents.items():
            straux += (x[0] + '-->[H: ' + str(x[1] / 3600) + ' M: ' + str(x[1] % 3600 / 60) + ' S: ' + str(x[1] % 3600 % 60) + ']\n')

        return straux

    def __str__ (self):

        straux = 'Blackboard: ' + '\n'

        for x in self.Registers.values():
            straux = straux + 'IdWork: ' + str(x.IdWork) + ' WorkUnit: ' + str(x.WorkUnit) + ' Size: ' + str(x.Size) + ' Comp: ' + str(x.Comp) + ' Test: ' + str(x.Test) + ' Treal: ' + str(x.Treal) + ' Agent: ' + str(x.Agent) + ' State: ' + x.State.__str__() + ' Ibs: ' + str(x.Ibs) + ' Ls: ' + str(x.Ls) + ' Rl: ' + str(x.Rl) + '\n'

        return straux

class Blackboard(Ice.Application):

    def run(self, argv):
        self.shutdownOnInterrupt()
        
        adapter = self.communicator().createObjectAdapter("AgentAdapter")
        adapter.activate()
        
        id = self.communicator().stringToIdentity(argv[1])
        agentI = BlackboardI(argv[1])
        proxy = adapter.add(agentI, id)
        agentPrx = FIPA.RenderAgentPrx.uncheckedCast(proxy)
        print 'Starting up the Master ' + agentPrx.ice_toString()

        prx = self.communicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession(argv[1], 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.addObjectWithType(agentPrx, agentPrx.ice_id())
        except IceGrid.ObjectExistsException:
            print agentPrx.ice_toString() + 'exists.'

        prx2 = self.communicator().stringToProxy('FIPA/Query')
        queryPrx = IceGrid.QueryPrx.checkedCast(prx2)
        obj2 = queryPrx.findObjectById(self.communicator().
                                       stringToIdentity('DF/topic.DirectoryFacilitator'))
        topic = IceStorm.TopicPrx.uncheckedCast(obj2)
        
        try:
            topic.subscribeAndGetPublisher(None, proxy)
        except IceStorm.NoSuchTopic:
            print 'The topic DF/topic.DirectoryFacilitator does not exist.'

        obj4 = queryPrx.findObjectById(self.communicator().stringToIdentity('df'))
        dfPrx = FIPA.DirectoryFacilitatorPrx.uncheckedCast(obj4)
        if dfPrx:
            description = FIPA.DfAgentDescription()
            description.proxy = agentPrx
            agentDesc = FIPA.AgentDescription()
            agentDesc.leaseTime = 0
            s = []
            s.append(FIPA.ServiceDescription('RenderAgent', 'magarro'))            
            agentDesc.services = s
            description.desc = agentDesc
            dfPrx.register(description)            

        obj3 = queryPrx.findObjectById(self.communicator().stringToIdentity('acc'))
        accPrx = FIPA.AgentCommunicationChannelPrx.uncheckedCast(obj3)
        print agentPrx.ice_toString() + ' is online.'

        self.communicator().waitForShutdown()

        topic.unsubscribe(proxy)
        agentI.clear(None)

        return 0

Blackboard().main(sys.argv, '../../../config/config.agent')
