#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt', 'TheSecondWorldHeaven']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

if HOST not in LOCAL_HOSTS:
        sys.path.append('/usr/local/IcePy-3.2.0/python')

import Ice, Glacier2

if HOST not in LOCAL_HOSTS:
        Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/Magarro.ice')
else:
        Ice.loadSlice('--all -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/Magarro.ice')

import FIPA
import Util, array

DEFAULT_LEVEL = 3
DEFAULT_OPTIMIZATION_LEVEL = 3
DEFAULT_WORK_NAME = 'work'

class Boss (Ice.Application):

    def getAnalystArguments (self, argv):

        level, optimizationLevel, work, workName = DEFAULT_LEVEL, DEFAULT_OPTIMIZATION_LEVEL, '', DEFAULT_WORK_NAME

        for i in range(1, len(argv)):

            # -p ==> Division level (1-3).
            if argv[i] == '-p':
                try:
                    level = int(argv[i + 1])
                except IndexError:
                    level = DEFAULT_LEVEL
                except ValueError:
                    level = DEFAULT_LEVEL

            # -o ==> Optimization level (1-5).
            if argv[i] == '-o':
                try:
                    optimizationLevel = int(argv[i + 1])
                except IndexError:
                    optimizationLevel = DEFAULT_OPTIMIZATION_LEVEL
                except ValueError:
                    optimizationLevel = DEFAULT_OPTIMIZATION_LEVEL                    

            # -w ==> Work file (.zip).
            if argv[i] == '-w':
                try:
                    work = argv[i + 1]
                    l = work.split('.')
                    if l[1] != 'zip':
                        print 'The file must be in .zip format.'
                        sys.exit(0)
                except:
                    print 'The file is not valid.'
                    sys.exit(0)

            # -n ==> Work name.
            if argv[i] == '-n':
                try:
                    workName = argv[i + 1]
                except:
                    print 'Using ' + DEFAULT_WORK_NAME + ' as the work name.'

        if work == '':
            print 'Must specify the work file.'
            print 'Synopsis: python Boss.py -i <file.xml> [- p <level>] -w <path_to_workfile> [-n <work_name>]'
            sys.exit(0)

        return level, optimizationLevel, work, workName

    def run (self, argv):

        self.shutdownOnInterrupt()

        level, optimizationLevel, work, workName = self.getAnalystArguments(argv)

        # Send the work to the analyst.
        obj = self.communicator().stringToProxy('analyst')
        prx = FIPA.AnalystPrx.uncheckedCast(obj)
        id = prx.processWork(work, workName, level, optimizationLevel)
        prx.notifyNewWorkToMaster(id, optimizationLevel)

	self.communicator().waitForShutdown()
        
	return 0

Boss().main(sys.argv, MAGARRO_HOME + '/../../../config/config.agent')
