#!/usr/bin/env python

import Image
import PIL.Image
import sys

def newImage(filename):
    return PIL.Image.open(filename)

def getImagePixels(image):
    if image.tell() != 0:
        image.seek(0)
    return image.load()

def getTotalPixels(hist):
    n = 0
    for i in hist:
        n += i

    return n

def histogramToProbDist(hist):
    pr = []
    npixels = getTotalPixels(hist)

    for i in range(len(hist)):
        pr.append(float(hist[i])/npixels)

    return pr

def getEqMatchingVector(img):
    eqhist = []
    hist = img.histogram()
    rhist = hist[:256]    # Histogram for R channel
    ghist = hist[256:512] # Histogram for G channel
    bhist = hist[512:768] # Histogram for B channel

    # Obtain probability distributions for each histogram
    rpr = histogramToProbDist(rhist)
    gpr = histogramToProbDist(ghist)
    bpr = histogramToProbDist(bhist)

    # Matching vector for R channel
    for i in range(len(rpr)):
        n = 0.0
        for j in range(i):
            n += rpr[j]
        eqhist.append(n + rpr[i])

    # Matching vector for G channel
    for i in range(len(gpr)):
        n = 0.0
        for j in range(i):
            n += gpr[j]
        eqhist.append(n + gpr[i])

    # Matching vector for B channel
    for i in range(len(bpr)):
        n = 0.0
        for j in range(i):
            n += bpr[j]
        eqhist.append(n + bpr[i])

    matvec = []
    for i in range(len(eqhist)):
        matvec.append(int(eqhist[i] * 255))

    return matvec


if __name__ == '__main__':
    if len(sys.argv) == 4:
        # Image to match
        img = newImage(sys.argv[1])
        eqmatvec = getEqMatchingVector(img)

        # Reference image
        masterimg = newImage(sys.argv[2])
        mastereqmatvec = getEqMatchingVector(masterimg)
        masterRChannel = mastereqmatvec[:256]
        masterGChannel = mastereqmatvec[256:512]
        masterBChannel = mastereqmatvec[512:768]

        # Start matching
        print "Performing histogram matching of %s against %s...." % (sys.argv[1], sys.argv[2]),
        sys.stdout.flush()
        result = PIL.Image.new('RGB', img.size)

        imgPix = getImagePixels(img)
        resultPix = getImagePixels(result)

        print imgPix         #########################################################
        print resultPix      #########################################################

        for i in range(img.size[0]):
            for j in range(img.size[1]):
                matchIndex = []
                pixvalue = imgPix[i, j]
                r = eqmatvec[pixvalue[0]]
                g = eqmatvec[pixvalue[1] + 256]
                b = eqmatvec[pixvalue[2] + 512]
                
                if r in masterRChannel:
                    matchIndex.append(masterRChannel.index(r))
                else:
                    found = False
                    n = 1
                    while not found and n < 255:
                        if (r - n) in masterRChannel:
                            found = True
                            r = r - n
                        elif (r + n) in masterRChannel:
                            found = True
                            r = r + n
                        else:
                            n += 1
                    matchIndex.append(masterRChannel.index(r))

                if g in masterGChannel:
                    matchIndex.append(masterGChannel.index(g))
                else:
                    found = False
                    n = 1
                    while not found and n < 255:
                        if (g - n) in masterGChannel:
                            found = True
                            g = g - n
                        elif (g + n) in masterGChannel:
                            found = True
                            g = g + n
                        else:
                            n += 1
                    matchIndex.append(masterGChannel.index(g))

                if b in masterBChannel:
                    matchIndex.append(masterBChannel.index(b))
                else:
                    found = False
                    n = 1
                    while not found and n < 255:
                        if (b - n) in masterBChannel:
                            found = True
                            b = b - n
                        elif (b + n) in masterBChannel:
                            found = True
                            b = b + n
                        else:
                            n += 1
                    matchIndex.append(masterBChannel.index(b))

                    
                resultPix[i,j] = tuple(matchIndex)

        result.save(sys.argv[3])
        print " done!"
        print "Result has been saved in %s" % (sys.argv[3],)

    else:
        print "  Usage: %s <image_to_match> <reference_image> <output_file>\n\n" % (sys.argv[0],)
