#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, array, zipfile, time

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt', 'TheSecondWorldHeaven']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

sys.path.append('/usr/local/IcePy-3.2.0/python')
import Image
import Ice, IceStorm, Glacier2, IceGrid
Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I../../../slice -I/usr/share/slice Magarro.ice')
import FIPA
sys.path.append('../../python')
import AgentI
from AgentI import *
import Bidding, RenderInfo, Util, Util_image
from Bidding import *
from RenderInfo import *

if HOST not in LOCAL_HOSTS:
    MASTER_WORK_DIR = MAGARRO_HOME + '/master'
    MASYRO_WEB_DIR = MAGARRO_HOME + "/gui"
    LOG = MASTER_WORK_DIR + '/log.txt'
else:
    MASTER_WORK_DIR = './master'
    MASYRO_WEB_DIR = "/var/www/magarro"
    LOG = MASTER_WORK_DIR + '/log.txt'
 
ASIGNED = 1
NOASIGNED = 0
WITHOUTPOSSIBILITY = -1
EXT = '.png'
OUTPUT = 'output.png'
 
class AMI_Model_renderI (object):
	
    def ice_response (self):
	print 'Render finished...'

    def ice_exception (self, ex):
	try:
	    raise ex
	except Ice.LocalException:
	    print 'Render error...', ex

class AMI_Model_notifyZonesI (object):
    def ice_response (self):
	print 'Notification done...'

    def ice_exception (self, ex):
	try:
	    raise ex
	except Ice.LocalException:
	    print 'Notification error...'

class MasterI (FIPA.Master, AgentI):
	
    def __init__ (self, name):
        AgentI.__init__(self, None)
	self.Name = name
	# Current work.
	self.CurrentZones, self.CurrentIdWork = [], -1
	self.CurrentImage = None
	self.InterpolationBand = True
	# (Key, Value) == (Agent name, proxy)
	self.RenderAgents = {}
	# Medium benchmark time in the platform.
	self.BenchmarkValue = 1
	# Current biddings.
	self.Biddings = []
	self.InitialBidding = 1
	# Number of current works received.
	self.NumberOfPieces = 0
	self.CurrentBiddingTime = 0
        # Estimation Time
        self.EstimationTime = 0
        # Optimization Level
        self.Optimization = 0

	# Proxies.
	self.Repository, self.Blackboard = None, None
	
	self.Log = ""
	self.FinalTimes = ""

    def getInternalStateOfAgent(self, agentName, curr = None):
	agentPrx = self.RenderAgents[agentName]
	return agentPrx.getInternalState()
	    
    def getAgentNames(self, curr = None):
	return self.RenderAgents.keys()

    def getNumberofAgents(self, curr):
        return len(self.RenderAgents)
        
    def getBlackboardState(self, curr):
	lregister = []
	for x in self.CurrentZones:
	    lregister.append(self.Blackboard.read (self.CurrentIdWork, x.id))
	return lregister
 
    def obtainProxies (self, comm):
	self.obtainRepository(comm)
	self.obtainBlackboard(comm)

    def obtainRepository (self, comm):

        try:
            obj = comm.stringToProxy('modelRepository')
            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)

            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(queryPrx.findObjectByType('::FIPA::ModelRepository'))
            
        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to the repository.'

    def obtainBlackboard (self, comm):
        """Obtiene un proxy a la pizarra"""

        try:
            obj = comm.stringToProxy('blackboard')
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)
            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(queryPrx.findObjectByType('::FIPA::ModelRepository'))

        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to the blackboard.'

    def subscribe (self, renderAgentName, renderAgent, curr):

        self._mutex.acquire()
        print self.Name + ' --> New agent subscribed: ' + renderAgentName + '.'
        self.RenderAgents[renderAgentName] = renderAgent
        self._mutex.release()

    def unsubscribe (self, renderAgentName, current = None):

        self._mutex.acquire()
        print self.Name + '--> The agent ' + renderAgentName + ' unsubscribed.'
        prx = self.RenderAgents.pop(renderAgentName)
        self._mutex.release()

    def clean (self, curr):

        self._mutex.acquire()
        for prx in self.RenderAgents.values():
            prx.destroy()
        self.RenderAgents = {}
        self._mutex.release()

    def benchmarkValue (self, value, curr):

        self._mutex.acquire()
        self.BenchmarkValue += value
        self._mutex.release()

    def notifyNewWork (self, zones, idWork, optimization, rendermethods, curr):

        self.Log = ''
        self.FinalTimes = ''
        self.Log += ('NewWork: ' + str(idWork) + '\n')
        self.CurrentBiddingTime = 0
        self.InitialBidding = 0
        self.Optimization = optimization

        self.obtainProxies(curr.adapter.getCommunicator())

        self.Blackboard.clear()
        self.Biddings = []

        self.CurrentZones = zones
        self.CurrentIdWork = idWork
        self.CurrentImage = Util_image.ImageSet()
        self.RenderMethods = rendermethods   # 1 = Path Tracing, 0 = Varios Metodos

        self.NumberOfPieces = 0

        # New work on the blackboard.
        self.writeWorkInBlackboard(idWork)
        # Notification.
        benchmarkMedium = 0
        try:
            benchmarkMedium = int(self.BenchmarkValue / len(self.RenderAgents))
        except ZeroDivisionError:
            print 'No agents subscribed.'

        self.notifyNewWorkToAgents(zones, idWork, benchmarkMedium)
        self.initialDistribution(optimization)

    def notifyNewWorkToAgents (self, zones, idWork, benchmarkValue):

        for prxAgent in self.RenderAgents.values():
            prxAgent.notifyNewWork(zones, idWork, benchmarkValue, self.RenderMethods)

    def initialDistribution (self, optimization):

        # Estimated render...
        zonesPerBlock = max(int(len(self.CurrentZones) / len(self.RenderAgents)), 1)
        
        for a in range(len(self.RenderAgents)):

            if a <> len(self.RenderAgents) - 1:
                zones = self.CurrentZones[a * zonesPerBlock: (a + 1) * zonesPerBlock]
            else:
                zones = self.CurrentZones[a * zonesPerBlock: len(self.CurrentZones)]

            agentPrx = self.RenderAgents.values()[a]
            cb = AMI_Model_notifyZonesI()
            agentPrx.notifyZones_async(cb, zones, self.CurrentIdWork, optimization)
        
        # The bid begins.
        self.beginRenderProcess()

    def beginRenderProcess (self):

        begin = time.time()
        # Wait until all agents finish their estimations.
        while True:
            if self.Blackboard.isWorkPartiallyEstimated():
                break
            else:
                time.sleep(3)
        end = time.time()
        self.EstimationTime = int(end-begin)
        self.Blackboard.stSetEstimationTime()

        # Final render.
        for agentPrx in self.RenderAgents.values():
            print agentPrx.ice_toString()
            agentPrx.beginRenderProcess()

    def bidHigher (self, agent, idWork, idZone, credits, historic, current = None):

        self._mutex.acquire()
        begin = time.time()
        
        # NOASIGNED shows the bid has not been confirmed yet.
        bidding = Bidding(agent, idWork, idZone, credits, historic, NOASIGNED)
        self.Biddings.append(bidding)
        self.Log += ('Bidding ==> ' + bidding.__str__() + '\n')
        # Zone distribution...
        print 'MASTER --> Biddings: '  + str(len(self.Biddings)) + ' Agents bidding: ' + str(self.getNumberFreeAgents())
        if self.InitialBidding < len(self.RenderAgents.values()):
            if len(self.Biddings) == self.getNumberFreeAgents():
                self.shareOutZones()
                self.InitialBidding += 1
        else:
            self.shareOutZones()

        end = time.time()
        self.CurrentBiddingTime += (int(end - begin))
        self._mutex.release()

    def getNumberFreeAgents (self):

        n = 0

        for agentPrx in self.RenderAgents.values():
            if agentPrx.getRenderState() == FIPA.StateRenderAgent.Bidding:
                n += 1

        return n

    def shareOutZones (self):

        print 'Zone distribution... Biddings: ' + str(len(self.Biddings))

        done = False
        rendersInfo = []

        # Iterate over the biddings...
        for i in range(len(self.Biddings)):
            if self.Biddings[i].State == NOASIGNED:
                best = i
                done = True

                # Compare the bidding i with the rest...
                for j in range(len(self.Biddings)):
                    # Same bidding, two agents.
                    idWork1, idWork2 = self.Biddings[i].IdWork, self.Biddings[j].IdWork
                    idZone1, idZone2 = self.Biddings[i].IdZone, self.Biddings[j].IdZone
                    state1, state2 = self.Biddings[i].State, self.Biddings[j].State

                    # If they bid for the same zone, it is assigned to the agent having the best offer.
                    if idWork1 == idWork2 and idZone1 == idZone2 and state2 == NOASIGNED:
                        # Best offer --> The biggest number of credits and the best historic.
                        v1 = self.Biddings[best].getCurrentHistoric()
                        v2 = self.Biddings[j].getCurrentHistoric()
                        v1 += self.Biddings[best].Credits
                        v2 += self.Biddings[j].Credits
                        if v1 < v2:
                            best = j

            # If some bid has done...
            if done:
                done = False
                # Assign the zone to the agent having the best offer.
                self.Biddings[best].State = ASIGNED
                # Descart the rest of bids.
                self.markWithoutPossibility(best)
                # Update the blackboard.
                a, b, c = self.CurrentIdWork, self.Biddings[best].IdZone, self.Biddings[best].Agent
                self.Blackboard.setWorkUnit(a, b, c)
                rendersInfo.append(RenderInfo(a, b, c))

        # Update for the next round.
        self.markWithPossibility()

        # Order the renders and the biddings to the agents.
        for b in self.Biddings:
            if self.isInListOfRenders(b.Agent, rendersInfo):
                self.Log += ('Rendering ==> ' + b.__str__() + '\n')
                self.orderRender(b.IdWork, b.IdZone, b.Agent)
            else:
                self.orderNewBidding(b.Agent)

        # Delete the previous bids.
        self.Biddings = []

    def isInListOfRenders (self, agent, rendersInfo):

        for r in rendersInfo:
            if r.Agent == agent:
                return True

        return False

    def markWithoutPossibility (self, best):

        for x in self.Biddings:
            if x.IdWork == self.Biddings[best].IdWork and x.IdZone == self.Biddings[best].IdZone and x.State == NOASIGNED:
                x.State = WITHOUTPOSSIBILITY

    def markWithPossibility (self):

        for x in self.Biddings:
            if x.State == WITHOUTPOSSIBILITY:
                x.State = NOASIGNED

    def orderRender (self, idWork, idZone, agent):

        agentPrx = self.RenderAgents[agent]
        cb = AMI_Model_renderI()

        if (self.RenderMethods == 1):
            agentPrx.render_async(cb, idWork, idZone, agent, 'PT')
        else:
            reg = self.Blackboard.read (idWork, idZone)
            TEstPT = reg.Test
            TEstRT = reg.TestRT
            TEstRC = reg.TestRC
            Comp = reg.Comp
            QDRT = reg.QDRT
            QDRC = reg.QDRC
            rm = 'PT'
            if (self.Optimization == 1):
                if (QDRT < 1):
                    if (TEstPT > TEstRT * 1.10):
                        rm = 'RT'
                    if (QDRC < QDRT) and (Comp < 50):
                        if (TEstPT > TEstRC * 1.20):
                            rm = 'RC'
            if (self.Optimization == 2):
                if (QDRT < 5):
                    if (TEstPT > TEstRT * 1.10):
                        rm = 'RT'
                    if ((QDRC < QDRT) and (Comp < 50)):
                        if (TEstPT > TEstRC * 1.20):
                            rm = 'RC'
            if (self.Optimization == 3):
                if (QDRT < 10):
                    if (TEstPT > TEstRT * 1.30):
                        rm = 'RT'
                    if ((QDRC < QDRT) and (Comp < 50)):
                        if (TEstPT > TEstRC * 1.40):
                            rm = 'RC'
            if (self.Optimization == 4):
                if (QDRT < 20):
                    if (TEstPT > TEstRT * 1.30):
                        rm = 'RT'
                    if ((QDRC < QDRT) and (Comp < 50)):
                        if (TEstPT > TEstRC * 1.40):
                            rm = 'RC'
            if (self.Optimization == 5):
                if (QDRT < 30):
                    if (TEstPT > TEstRT * 1.30):
                        rm = 'RT'
                    if ((QDRC < QDRT) and (Comp < 50)):
                        if (TEstPT > TEstRC * 1.40):
                            rm = 'RC'

            self.Blackboard.updateMethod(idWork, idZone, rm)
            agentPrx.render_async(cb, idWork, idZone, agent, rm)

    def orderNewBidding (self, agent):

        agentPrx = self.RenderAgents[agent]
        agentPrx.beginRenderProcess()

    def noMoreBiddings (self, current = None):

        self._mutex.acquire()
        if self.Blackboard.isCurrentWorkFinished():
            self.Log += ('\n\n' + self.Blackboard.show() + '\n\n')
            self.FinalTimes += ('AnalysisTime: ' + str(self.Blackboard.getAnalysisTime()) + '\n')
            self.FinalTimes += ('EstimatedRenderTime: ' + str(self.Blackboard.getEstimatedRenderTime()) + '\n')
            self.FinalTimes += ('BiddingTime: ' + str(self.CurrentBiddingTime) + '\n\n')
            self.FinalTimes += self.Blackboard.getTimeCurrentWorks()
            f = open(LOG, 'w')
            f.write(self.Log)
            f.write(self.FinalTimes)
            f.close()

            for x in self.RenderAgents.values():
                print x.getRenderState()

            self.Blackboard.stSetBiddingTime(int(self.CurrentBiddingTime))
            self.Blackboard.stSetRenderTime()
        self._mutex.release()

    def giveFinalImage (self, idWork, idZone, partialImage, x1, y1, x2, y2, ibs, current = None):

        # Store the received part as an image.
        path = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(idZone) + EXT)
        array.array('B', partialImage).tofile(open(path, 'w'))

        # Update the information of the final result.
        self.CurrentImage.add(idZone, Util_image.PieceOfImage(x1, y1, x2, y2, ibs))
        self.NumberOfPieces += 1

        finalImage = Image.open(path)
        pathF = os.path.join(MASTER_WORK_DIR, OUTPUT)
        finalImage.save(pathF)
        resX, resY = finalImage.size

        # Guess the number of neighbours of each pieceOfImage.
        for i in self.CurrentImage.Images.values():
            i.ResX = resX
            i.ResY = resY
            self.CurrentImage.setNeighbourhood(i)

        self.finalComposition2(idWork, finalImage)

    def setInterpolationBand(self, enableBand, current = None):
        self.InterpolationBand = enableBand

    def getInterpolationBand(self, current = None):
        return self.InterpolationBand

    def finalComposition (self, idWork, finalImage):

        for x in self.CurrentImage.Images.items():
            path = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
            im = Image.open(path)
            finalIbs = x[1].Ibs
            box = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
            region = im.crop(box)
            finalImage.paste(region, box)

        if self.InterpolationBand:
            for x in self.CurrentImage.Images.items():
                for y in self.CurrentImage.Images.items():
                    
                    # Se comprueba si y es el vecino derecho de x.
                    t, limite = x[1].isRightNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        # Limitex representa el trozo a interpolar de x.
                        #limitex = (limite[0] - finalIbs, limite[1], limite[2] + finalIbs, limite[3])
                        # Limitey representa el trozo a interpolar de y.
                        #limitey = (limite[0], limite[1], limite[2] + finalIbs, limite[3])
                        #limitey = (limite[0], limite[1], limite[2] + y[1].Ibs, limite[3])
                        # LimiteFinal representa todo el trozo a interpolar.
                        #limiteFinal = (limitex[0], limitex[1], limitey[2], limitey[3])
                        #limiteFinal = (limitex[0] - x[1].Ibs, limitex[1], limitey[2] + y[1].Ibs, limitey[3])
                        limiteFinal = (limite[0] - finalIbs, limite[1], limite[2] + finalIbs, limite[3])
              	        #print 'R: ', limiteFinal

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        mask = Util_image.createMask(limiteFinal[2] - limiteFinal[0], limiteFinal[3] - limiteFinal[1])
                        im3 = Image.composite(imI, imD, mask)
                        finalImage.paste(im3, limiteFinal)

                    # Se comprueba si y es el vecino izquierdo de x.
                    t, limite = x[1].isLeftNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        # Limitex representa el trozo a interpolar de x.
                        #limitex = (limite[0], limite[1], limite[2] + finalIbs, limite[3])
                        #limitex = (limite[0], limite[1], limite[2] + x[1].Ibs, limite[3])
                        # Limitey representa el trozo a interpolar de y.
                        #limitey = (limite[0] - finalIbs, limite[1], limite[2], limite[3])
                        #limitey = (limite[0] - y[1].Ibs, limite[1], limite[2], limite[3])
                        # LimiteFinal representa todo el trozo a interpolar.
                        #limiteFinal = (limitey[0], limitey[1], limitex[2], limitex[3])
                        limiteFinal = (limite[0] - finalIbs, limite[1], limite[2] + finalIbs, limite[3])
                        #print 'L: ', limiteFinal

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        mask = Util_image.createMask(limiteFinal[2] - limiteFinal[0], limiteFinal[3] - limiteFinal[1])
                        im3 = Image.composite(imI, imD, mask)
                        finalImage.paste(im3, limiteFinal)

                    # Se comprueba si y es el vecino superior de x.
                    t, limite = x[1].isUpNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        # Limitex representa el trozo a interpolar de x.
                        #limitex = (limite[0], limite[1], limite[2], limite[3] + finalIbs)
                        #limitex = (limite[0], limite[1], limite[2], limite[3] + x[1].Ibs)
                        # Limitey representa el trozo a interpolar de y.
                        #limitey = (limite[0] - finalIbs, limite[1], limite[2], limite[3])
                        #limitey = (limite[0] - y[1].Ibs, limite[1], limite[2], limite[3])
                        # LimiteFinal representa todo el trozo a interpolar.
                        #limiteFinal = (limitey[0], limitey[1] - finalIbs, limitex[2], limitex[3])
                        #limiteFinal = (limitey[0], limitey[1], limitex[2], limitex[3])
                        #limiteFinal = (limitey[0], limitey[1] - y[1].Ibs, limitex[2], limitex[3] + x[1].Ibs)
                        limiteFinal = (limite[0], limite[1] - finalIbs, limite[2], limite[3] + finalIbs)
                        #print 'U: ', limiteFinal

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        mask = Util_image.createMask(limiteFinal[2] - limiteFinal[0], limiteFinal[3] - limiteFinal[1])
                        im3 = Image.composite(imI, imD, mask)
                        finalImage.paste(im3, limiteFinal)

                    # Se comprueba si y es el vecino inferior de x.
                    t, limite = x[1].isDownNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)                        
                        # Limitex representa el trozo a interpolar de x.
                        #limitex = (limite[0], limite[1], limite[2], limite[3] + finalIbs)
                        #limitex = (limite[0] - x[1].Ibs, limite[1] - x[1].Ibs, limite[2] + x[1].Ibs, limite[3] + x[1].Ibs)
                        # Limitey representa el trozo a interpolar de y.
                        #limitey = (limite[0], limite[1] - finalIbs, limite[2], limite[3])
                        #limitey = (limite[0] - y[1].Ibs, limite[1] - y[1].Ibs, limite[2] + y[1].Ibs, limite[3] + y[1].Ibs)
                        # LimiteFinal representa todo el trozo a interpolar.
                        #limiteFinal = (limitey[0], limitey[1], limitex[2], limitex[3] + finalIbs)
                        #limiteFinal = (limitex[0], limitex[1], limitey[2], limitey[3])
                        #limiteFinal = (limitex[0], limitex[1], limitey[2], limitey[3])
                        limiteFinal = (limite[0], limite[1] - finalIbs, limite[2], limite[3] + finalIbs)
                        #print 'D: ', limiteFinal

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        mask = Util_image.createMask(limiteFinal[2] - limiteFinal[0], limiteFinal[3] - limiteFinal[1])
                        im3 = Image.composite(imI, imD, mask)
                        finalImage.paste(im3, limiteFinal)                                                                        

        # Store the final image.
        pathF = os.path.join(MASTER_WORK_DIR, OUTPUT)
        finalImage.save(pathF)
        Util_image.createLowResolutionImage(MASTER_WORK_DIR, OUTPUT, MASYRO_WEB_DIR)

    def finalComposition2 (self, idWork, finalImage):

        for x in self.CurrentImage.Images.items():
            path = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
            im = Image.open(path)
            finalIbs = x[1].Ibs
            box = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
            region = im.crop(box)
            finalImage.paste(region, box)

        if self.InterpolationBand:
            for x in self.CurrentImage.Images.items():
                for y in self.CurrentImage.Images.items():
                    
                    # Se comprueba si y es el vecino derecho de x.
                    t, limite = x[1].isRightNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        limiteFinal = (limite[0] - finalIbs, limite[1], limite[2] + finalIbs, limite[3])

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        imI.save('/tmp/imI.png')
                        imD.save('/tmp/imD.png')
                        w = limiteFinal[2] - limiteFinal[0]
                        h = limiteFinal[3] - limiteFinal[1]

                        command = 'convert -size ' + str(h) + 'x' + str(w) + ' gradient: -rotate -90 /tmp/gradient.jpg'
                        os.system(command)
                        command2 = 'convert /tmp/imI.png /tmp/gradient.jpg +matte -compose Copy_Opacity -composite /tmp/imI2.png'
                        os.system(command2)
                        command3 = 'composite -compose ATop /tmp/imI2.png /tmp/imD.png /tmp/out.png'
                        os.system(command3)

                        im3 = Image.open('/tmp/out.png')
                        finalImage.paste(im3, limiteFinal)
                        
                    # Se comprueba si y es el vecino izquierdo de x.
                    t, limite = x[1].isLeftNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        limiteFinal = (limite[0] - finalIbs, limite[1], limite[2] + finalIbs, limite[3])

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        imI.save('/tmp/imI.png')
                        imD.save('/tmp/imD.png')
                        w = limiteFinal[2] - limiteFinal[0]
                        h = limiteFinal[3] - limiteFinal[1]

                        command = 'convert -size ' + str(h) + 'x' + str(w) + ' gradient: -rotate -90 /tmp/gradient.jpg'
                        os.system(command)
                        command2 = 'convert /tmp/imI.png /tmp/gradient.jpg +matte -compose Copy_Opacity -composite /tmp/imI2.png'
                        os.system(command2)
                        command3 = 'composite -compose ATop /tmp/imI2.png /tmp/imD.png /tmp/out.png'
                        os.system(command3)

                        im3 = Image.open('/tmp/out.png')
                        finalImage.paste(im3, limiteFinal)

                    # Se comprueba si y es el vecino superior de x.
                    t, limite = x[1].isUpNeighbour(y[1])
                    if t:

                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)
                        limiteFinal = (limite[0], limite[1] - finalIbs, limite[2], limite[3] + finalIbs)

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        imI.save('/tmp/imI.png')
                        imD.save('/tmp/imD.png')
                        w = limiteFinal[2] - limiteFinal[0]
                        h = limiteFinal[3] - limiteFinal[1]

                        command = 'convert -size ' + str(w) + 'x' + str(h) + ' gradient: /tmp/gradient.jpg'
                        os.system(command)
                        command2 = 'convert imI.png /tmp/gradient.jpg +matte -compose Copy_Opacity -composite /tmp/imI2.png'
                        os.system(command2)
                        command3 = 'composite -compose ATop /tmp/imI2.png /tmp/imD.png /tmp/out.png'
                        os.system(command3)

                        im3 = Image.open('/tmp/out.png')
                        finalImage.paste(im3, limiteFinal)

                    # Se comprueba si y es el vecino inferior de x.
                    t, limite = x[1].isDownNeighbour(y[1])
                    if t:
                        
                        path1 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(x[0]) + EXT)
                        path2 = os.path.join(MASTER_WORK_DIR, str(idWork) + '_' + str(y[0]) + EXT)
                        im1 = Image.open(path1)
                        im2 = Image.open(path2)
                        # finalIbs es el valor final de la banda de interpolación.
                        finalIbs = min(x[1].Ibs, y[1].Ibs)                        
                        # Limitex representa el trozo a interpolar de x.
                        limiteFinal = (limite[0], limite[1] - finalIbs, limite[2], limite[3] + finalIbs)

                        # Se pegan los dos trozos.
                        boxX = (x[1].X1, x[1].Y1, x[1].X2, x[1].Y2)
                        regionX = im1.crop(boxX)
                        boxY = (y[1].X1, y[1].Y1, y[1].X2, y[1].Y2)
                        regionY = im2.crop(boxY)
                        finalImage.paste(regionX, boxX)
                        finalImage.paste(regionY, boxY)

                        # Se actualiza la vecindad de los trozos.
                        imI = im1.crop(limiteFinal)
                        imD = im2.crop(limiteFinal)
                        imI.save('/tmp/imI.png')
                        imD.save('/tmp/imD.png')
                        w = limiteFinal[2] - limiteFinal[0]
                        h = limiteFinal[3] - limiteFinal[1]

                        command = 'convert -size ' + str(w) + 'x' + str(h) + ' gradient: /tmp/gradient.jpg'
                        os.system(command)
                        command2 = 'convert /tmp/imI.png /tmp/gradient.jpg +matte -compose Copy_Opacity -composite /tmp/imI2.png'
                        os.system(command2)
                        command3 = 'composite -compose ATop /tmp/imI2.png /tmp/imD.png /tmp/out.png'
                        os.system(command3)

                        im3 = Image.open('/tmp/out.png')
                        finalImage.paste(im3, limiteFinal)                                                                        
                        

        # Store the final image.
        pathF = os.path.join(MASTER_WORK_DIR, OUTPUT)
        finalImage.save(pathF)
        Util_image.createLowResolutionImage(MASTER_WORK_DIR, OUTPUT, MASYRO_WEB_DIR)                

    def showAgentStates (self, current = None):

        straux = ''

        for agent in self.RenderAgents.items():
            straux += (agent[0] + ':' + str(agent[1].getRenderState()) + '\n')

        return straux

    def getLog (self, current = None):
        return self.Log

    def getFinalTimes (self, current = None):
        return self.FinalTimes

    def writeWorkInBlackboard (self, idWork):

        # Write the registers associated to the zones.
        for x in self.CurrentZones:
            size = (x.y2 - x.y1) * (x.x2 - x.x1)
            register = FIPA.TRegister(idWork, x.id, int(size), int(x.m), 0, 0, 0, 0.0, 0.0, '', 0, '', FIPA.StateRegister.NotDone, 0, 0, 0)
            self.Blackboard.write(register)

    def clear (self):

        os.system('rm -f ' + LOG)

class Master(Ice.Application):

    def run(self, argv):
        self.shutdownOnInterrupt()
        
        adapter = self.communicator().createObjectAdapter("AgentAdapter")
        adapter.activate()
        
        id = self.communicator().stringToIdentity(argv[1])
        agentI = MasterI(argv[1])
        proxy = adapter.add(agentI, id)
        agentPrx = FIPA.RenderAgentPrx.uncheckedCast(proxy)
        print 'Starting up the Master ' + agentPrx.ice_toString()

        prx = self.communicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession(argv[1], 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.addObjectWithType(agentPrx, agentPrx.ice_id())
        except IceGrid.ObjectExistsException:
            print agentPrx.ice_toString() + 'exists.'

        prx2 = self.communicator().stringToProxy('FIPA/Query')
        queryPrx = IceGrid.QueryPrx.checkedCast(prx2)
        obj2 = queryPrx.findObjectById(self.communicator().
                                       stringToIdentity('DF/topic.DirectoryFacilitator'))
        topic = IceStorm.TopicPrx.uncheckedCast(obj2)
        
        try:
            topic.subscribeAndGetPublisher(None, proxy)
        except IceStorm.NoSuchTopic:
            print 'The topic DF/topic.DirectoryFacilitator does not exist.'

        obj4 = queryPrx.findObjectById(self.communicator().stringToIdentity('df'))
        dfPrx = FIPA.DirectoryFacilitatorPrx.uncheckedCast(obj4)
        if dfPrx:
            description = FIPA.DfAgentDescription()
            description.proxy = agentPrx
            agentDesc = FIPA.AgentDescription()
            agentDesc.leaseTime = 0
            s = []
            s.append(FIPA.ServiceDescription('RenderAgent', 'magarro'))            
            agentDesc.services = s
            description.desc = agentDesc
            dfPrx.register(description)            

        obj3 = queryPrx.findObjectById(self.communicator().stringToIdentity('acc'))
        accPrx = FIPA.AgentCommunicationChannelPrx.uncheckedCast(obj3)
        print agentPrx.ice_toString() + ' is online.'

        # SPECIFIC
        agentI.obtainProxies(self.communicator())
        # END_OF_SPECIFIC

        self.communicator().waitForShutdown()

        topic.unsubscribe(proxy)
        agentI.clear()

        return 0

Master().main(sys.argv, '../../../config/config.agent')
