#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, array 
sys.path.append('/usr/local/IcePy-3.2.0/python')
import Ice, IceGrid, IceStorm
###
## Ice.loadSlice('--all -I../../../slice -I. -I/usr/share/slice Magarro.ice')
###
Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I../../../slice -I/usr/share/slice Magarro.ice')
import FIPA
sys.path.append('../../python')
import AgentI
from AgentI import *
import Util

MODEL_WORK_DIR = './models'
EXT = '.zip'

class ModelRepositoryI (FIPA.ModelRepository, AgentI):

    def __init__ (self, name):

        AgentI.__init__(self, None)
        self.Name = name
        self.Models = {}
        self.NumberOfModels = 0

    def addModel (self, name):

        self.NumberOfModels += 1
        self.Models[self.NumberOfModels] = name

    def getModelName (self, idModel):

        if self.Models.has_key(idModel):
            return self.Models[idModel]
        else:
            raise KeyError

    def put (self, name, model, curr):

        print 'ModelRepository --> Storing ' + name

        path = os.path.join(MODEL_WORK_DIR, name)
        array.array('B', model).tofile(open(path + EXT, 'w'))

        # Store the model.
        self.addModel(name)

        # Return the id related to the model.
        return self.NumberOfModels

    def get (self, idModel, curr):

        print self.Name + ' --> Accessing to the model repository...'
        try:
            name = self.getModelName(idModel)
        except KeyError:
            raise FIPA.ModelNotExistsException('The model specified does not exist', self.Models.keys())

        path = os.path.join(MODEL_WORK_DIR, name + EXT)
        model = array.array('B', open(path).read()).tolist()

        return name, model

    def putWU (self, name, model, curr):
        print 'ModelRepository --> Storing WU ' + name
        path = os.path.join(MODEL_WORK_DIR, name)
        array.array('B', model).tofile(open(path, 'w'))

    def getWU (self, name, curr):
        print self.Name + ' --> Accessing to the model repository for WU...'
        path = os.path.join(MODEL_WORK_DIR, name)
        model = array.array('B', open(path).read()).tolist()
        return model


class ModelRepository(Ice.Application):

    def run(self, argv):
        self.shutdownOnInterrupt()
        
        adapter = self.communicator().createObjectAdapter("AgentAdapter")
        adapter.activate()
        
        id = self.communicator().stringToIdentity(argv[1])
        agentI = ModelRepositoryI(argv[1])
        proxy = adapter.add(agentI, id)
        agentPrx = FIPA.RenderAgentPrx.uncheckedCast(proxy)
        print 'Starting up the ModelRepository ' + agentPrx.ice_toString()

        prx = self.communicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession(argv[1], 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.addObjectWithType(agentPrx, agentPrx.ice_id())
        except IceGrid.ObjectExistsException:
            print agentPrx.ice_toString() + 'exists.'

        prx2 = self.communicator().stringToProxy('FIPA/Query')
        queryPrx = IceGrid.QueryPrx.checkedCast(prx2)
        obj2 = queryPrx.findObjectById(self.communicator().
                                       stringToIdentity('DF/topic.DirectoryFacilitator'))
        topic = IceStorm.TopicPrx.uncheckedCast(obj2)
        
        try:
            topic.subscribeAndGetPublisher(None, proxy)
        except IceStorm.NoSuchTopic:
            print 'The topic DF/topic.DirectoryFacilitator does not exist.'

        obj4 = queryPrx.findObjectById(self.communicator().stringToIdentity('df'))
        dfPrx = FIPA.DirectoryFacilitatorPrx.uncheckedCast(obj4)
        if dfPrx:
            description = FIPA.DfAgentDescription()
            description.proxy = agentPrx
            agentDesc = FIPA.AgentDescription()
            agentDesc.leaseTime = 0
            s = []
            s.append(FIPA.ServiceDescription('ModelRepository', 'magarro'))            
            agentDesc.services = s
            description.desc = agentDesc
            dfPrx.register(description)            

        obj3 = queryPrx.findObjectById(self.communicator().stringToIdentity('acc'))
        accPrx = FIPA.AgentCommunicationChannelPrx.uncheckedCast(obj3)
        print agentPrx.ice_toString() + ' is online.'

        self.communicator().waitForShutdown()

        topic.unsubscribe(proxy)

        return 0

ModelRepository().main(sys.argv, '../../../config/config.agent')

