#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, array, zipfile, time, math

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt', 'TheSecondWorldHeaven']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

if HOST not in LOCAL_HOSTS:
	sys.path.append('/usr/local/IcePy-3.2.0/python')

import Ice, IceStorm

if HOST not in LOCAL_HOSTS:
	Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/Magarro.ice')
else:
	Ice.loadSlice('--all -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/Magarro.ice')

import FIPA
sys.path.append(MAGARRO_HOME + '/../../python')

import AgentI
from AgentI import *
import Zone
from Zone import *
import Util_fuzzy, Util, Benchmark
from Util_fuzzy import *
from Util_image import *

RES = 10 # % of resolution decrease.
INITIAL_CREDITS = 10
PRIZE, FAILURE = 2, -1
GOODRESULT, BADRESULT = 1, 0
# Adjustment of the intern coefficient.
ADJUST = 0.1
# Number of partitions of the fuzzy sets.
NUMPARTSCOMPLEXITY, NUMPARTSNEIGHBOUR, NUMPARTSSIZE, NUMPARTSOPTIMIZATION = 5, 5, 3, 5
INPUTFUZZY = MAGARRO_HOME + '/fuzzyMasyro.xml'
INITDIR = MAGARRO_HOME
INFINITO, MENOSINFINITO = 999999, -999999
# Weight assigned to Test to estimate the complexity of a zone.
TEST_WEIGHT = 0.75
EXT = '.png'

class AMI_Model_bidHigherI (object):

    def ice_response (self):
        print 'Bid finished...'

    def ice_exception (self, ex):
        try:
            raise ex
        except Ice.LocalException:
            print 'Se produjo un error al pujar...'

class RenderAgentI (FIPA.RenderAgent, AgentI):

    def __init__ (self, name):

        AgentI.__init__(self, None)
        self.Name = name
        # Current work.
        self.CurrentZones, self.CurrentIdWork = [], -1
        # Zone of current work.
        self.CurrentIdZone = None
        # Current files.
        self.CurrentFiles = []
        self.Credits = INITIAL_CREDITS
        # Historic of successes and failures.
        self.Historic = []
	# Zones Rendered by this Agent
	self.RenderedZones = []
        # Intern coefficient of the agent.
        self.Coefficient = 0.0
        # Benchmark time.
        self.BenchmarkValue = 0.0
        # State of the agent --> Estimating, Bidding, Rendering, Resting.
        self.State = FIPA.StateRenderAgent.Resting

        self.OptimizationLevel = 0
        self.FuzzySystem = None

        # Well-known agents.
        self.Master = None
        self.Repository = None
        self.Blackboard = None

        # Work directory of the agent.
	self.WorkDir = MAGARRO_HOME + '/' + self.Name
	#self.WorkDir = './' + self.Name
	os.system('mkdir ' + self.WorkDir)


    def getInternalState(self, curr):
	return (FIPA.TAgentState(self.Name, self.RenderedZones, self.CurrentIdWork, self.Credits, self.Historic, self.Coefficient, self.BenchmarkValue, self.State))	

    def obtainProxies (self, comm):

        self.obtainMaster(comm)
        self.obtainRepository(comm)
        self.obtainBlackboard(comm)

    def getZone (self, zone):

        for x in self.CurrentZones:
            if x.id == zone:
                return x

    def obtainMaster (self, comm):

        try:
            # Searching for a Master.
            self.Master = FIPA.MasterPrx.checkedCast(comm.stringToProxy('master'))
            return True

        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to a Master.'
            return False

    def obtainRepository (self, comm):

        try:
            obj = comm.stringToProxy('modelRepository')
            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)
            self.Repository = FIPA.ModelRepositoryPrx.checkedCast(queryPrx.findObjectByType('::FIPA::ModelRepository'))

        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to a Master.'

    def obtainBlackboard (self, comm):

        try:
            obj = comm.stringToProxy('blackboard')
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(obj)
            
        except Ice.NotRegisteredException:
            prx = comm.stringToProxy('FIPA/Query')
            queryPrx = IceGrid.QueryPrx.checkedCast(prx)
            self.Blackboard = FIPA.BlackboardPrx.checkedCast(queryPrx.findObjectByType('::FIPA::Blackboard'))

        except Ice.LocalException:
            print self.Name + ' --> was not able to obtain a proxy to a Blackboard.'                            
            
    def getRenderState (self, curr):
        return self.State

    def notifyNewWork (self, zones, idWork, benchmarkValue, rendermethods, curr):

        print 'New work available: ', idWork
        self.CurrentZones = zones
        self.CurrentIdWork = idWork
        self.BenchmarkValue = benchmarkValue
	self.RenderMethods = rendermethods   # 1 = PT, 0 = varios metodos
        self.work()

        print self.Name + ' --> Loading the fuzzy system...'
        self.loadFuzzySystem()

    def work (self):

        # Obtain the work model.
        name, model = self.Repository.get(self.CurrentIdWork)
       	
	###
	print self.WorkDir, name
 
        # Save the file received.
        path = os.path.join(self.WorkDir, name)
        array.array('B', model).tofile(open(path, 'w'))
        # Unzip.
        unzip = Util.Unzip(path)
        # Store the file names of the zip file.
        nameList = unzip.extract(self.WorkDir)
        self.CurrentFiles = nameList

    def notifyBenchmarkValue (self):
        self.Master.benchmarkValue(int(Benchmark.getValue()))

    def notifyZones (self, zones, idWork, optimization, curr):

        self.State = FIPA.StateRenderAgent.Estimating
        
        self.OptimizationLevel = optimization
        for z in zones:
            print self.Name + ' --> Estimating... IdWork: ' + str(idWork) + ' WorkUnit: ' + str(z.id)
            self.estimatedRender(idWork, z.id)

        self.State = FIPA.StateRenderAgent.Resting

    def estimatedRender (self, idWork, idZone):

        blenderFile = Util.getFileName(self.CurrentFiles, 'blend')
        ficheroBlender = os.path.join(self.WorkDir, blenderFile)
        
        salida = os.path.join(self.WorkDir, str(idWork) + '_' + str(idZone))
        zone = self.getZone(idZone)
        x1, y1 = zone.x1, zone.y1
        x2, y2 = zone.x2, zone.y2

	if (self.RenderMethods == 0):
	    begin = time.time()      # RAY CASTING
	    os.system('blender -b ' + ficheroBlender + ' -P ' + MAGARRO_HOME + '/EstimatedRenderRC.py -f 1 ' + salida + ' ' + str(x1) + ' ' + str(y1) + ' ' + str(x2) + ' ' + str(y2) + ' ' + str(RES))
	    end = time.time()
	    os.system('mv ' + salida + '0001.png ' + salida + 'RC' + EXT)
	    if len(self.Historic) == 0:
	        self.Coefficient = Benchmark.getValue() / self.BenchmarkValue
	    finalTime = int(RES * (end - begin) / self.Coefficient)
	    self.Blackboard.updateRC(idWork, idZone, finalTime)

	    begin = time.time()      # RAY TRACING
	    os.system('blender -b ' + ficheroBlender + ' -P ' + MAGARRO_HOME + '/EstimatedRenderRT.py -f 1 ' + salida + ' ' + str(x1) + ' ' + str(y1) + ' ' + str(x2) + ' ' + str(y2) + ' ' + str(RES))
	    end = time.time()
	    os.system('mv ' + salida + '0001.png ' + salida + 'RT' + EXT)
	    if len(self.Historic) == 0:
		self.Coefficient = Benchmark.getValue() / self.BenchmarkValue
	    finalTime = int(RES * (end - begin) / self.Coefficient)
	    self.Blackboard.updateRT(idWork, idZone, finalTime)

        begin = time.time()
        # Render with EstimatedRender.py.
        os.system('blender -b ' + ficheroBlender + ' -P ' + MAGARRO_HOME + '/EstimatedRender.py -f 1 ' + salida + ' ' + str(x1) + ' ' + str(y1) + ' ' + str(x2) + ' ' + str(y2) + ' ' + str(RES))
        end = time.time()

        os.system('mv ' + salida + '0001.png ' + salida + EXT)

        # Update the blackboard with the estimated time.
        self.Blackboard.incrementEstimatedRenderTime(int(end - begin))
        
        if len(self.Historic) == 0:
	    self.Coefficient = Benchmark.getValue() / self.BenchmarkValue
        finalTime = int(RES * (end - begin) / self.Coefficient)
        self.Blackboard.update(idWork, idZone, finalTime)

	if (self.RenderMethods == 0):
	    # Envia el fichero de PT al repositorio
            trabajo = array.array('B', open(salida + EXT).read()).tolist()
	    self.Repository.putWU(str(idWork) + '_' + str(idZone) + EXT, trabajo)
	    
	    # Actualizar el porcentaje de diferencia
	    # Ajustamos el histograma primero...
	    os.system('python HistoMatch.py '+ salida + 'RT' + EXT + ' ' + salida + EXT + ' ' + salida + 'RTHM' + EXT)
	    os.system('python HistoMatch.py '+ salida + 'RC' + EXT + ' ' + salida + EXT + ' ' + salida + 'RCHM' + EXT)
	    cropImage(salida + 'RTHM'+EXT, 'PNG', x1 * RES/100.0, y1 * RES/100.0, x2 * RES/100.0, y2 * RES/100.0)
	    cropImage(salida + 'RCHM'+EXT, 'PNG', x1 * RES/100.0, y1 * RES/100.0, x2 * RES/100.0, y2 * RES/100.0)
	    cropImage(salida + EXT, 'PNG', x1 * RES/100.0, y1 * RES/100.0, x2 * RES/100.0, y2 * RES/100.0)
	    percRT = meanDifference(salida + 'RTHM' + EXT, salida + EXT)
	    percRC = meanDifference(salida + 'RCHM' + EXT, salida + EXT)
	    self.Blackboard.updatePercRT(idWork, idZone, percRT)
	    self.Blackboard.updatePercRC(idWork, idZone, percRC)

    def beginRenderProcess (self, curr):

        print 'Rendering...'
        self.State = FIPA.StateRenderAgent.Bidding

        # Load the blackboard registers to bid.
        idMostComplexZone, maxComplex = -1, -1
        for x in self.CurrentZones:
            register = self.Blackboard.read(self.CurrentIdWork, x.id)
            # Most complex zone ==> 75% weight Test 25% weigh Comp.
            # If the zone is more complex than the current zone and is free...
            maxTest = self.Blackboard.getMaxTest(self.CurrentIdWork)
            maxComp = self.Blackboard.getMaxComp(self.CurrentIdWork)
            currentZoneComplexity = self.getComplexityRatio(register.Test, maxTest, register.Comp, maxComp)
            if currentZoneComplexity > maxComplex and register.Agent == '':
                idMostComplexZone = x.id
                maxComplex = currentZoneComplexity

        # Bid for the most complex zone.
        if idMostComplexZone != -1:
            
            print self.Name + ' --> Bidding for ' + str(self.CurrentIdWork) + ' ' + str(idMostComplexZone)
            cb = AMI_Model_bidHigherI()
            self.Master.bidHigher_async(cb, self.Name, self.CurrentIdWork, idMostComplexZone, self.Credits, self.Historic)
        # The agent notifies the Master if it is not able to bid.
        else:
            print self.Name + ' --> I am not able to bid.'
            self.State = FIPA.StateRenderAgent.Finishing
            self.Master.noMoreBiddings()

    def getComplexityRatio (self, test, maxTest, comp, maxComp):
        return TEST_WEIGHT * (float(test * 100 / maxTest)) + (1 - TEST_WEIGHT) * (float(comp * 100 / maxComp))

    def render (self, idWork, idZone, agent, method, curr):

        self.State = FIPA.StateRenderAgent.Rendering

        # Update the current work zone.
        self.CurrentIdZone = idZone
        # Rendering...
        finalTime = self.finalRender(self.OptimizationLevel, method)

        self.State = FIPA.StateRenderAgent.Resting

        # Adjustment process.
        self.internAdjustment(int(finalTime / self.Coefficient))

        # Bid for a new work.
        self.beginRenderProcess(None)

    def finalRender (self, optimizationLevel, method):

        blenderFile = Util.getFileName(self.CurrentFiles, 'blend')
        ficheroBlender = os.path.join(self.WorkDir, blenderFile)
        salida = os.path.join(self.WorkDir, str(self.CurrentIdWork) + '_' + str(self.CurrentIdZone))

        # Get the zone.
        zone = self.getZone(self.CurrentIdZone)
        x1, y1 = zone.x1, zone.y1
        x2, y2 = zone.x2, zone.y2
        aux = ' Zone: [' + str(x1) + ', ' + str(y1) + ', ' + str(x2) + ', ' + str(y2) + ']'
        print 'Rendering... ' + str(self.CurrentIdWork) + ' ' + str(self.CurrentIdZone) + aux

        # Obtain the output parameters of the fuzzy system.
        data = self.loadDataFuzzySystem(optimizationLevel)
        rl, ibs, ls = -1, -1, -1
        for x in data.keys():
            if x == 'Rl':
                rl = data[x]
            elif x == 'Ibs':
                ibs = data[x]
            elif x == 'Ls':
                ls = data[x]

        # Get the time to update the blackboard.
        begin = time.time()
        os.system('blender -b ' + ficheroBlender + ' -P ' + MAGARRO_HOME + '/FinalRender'+ method +'.py -f 1 ' + salida + ' ' + str(x1) + ' ' + str(y1) + ' ' + str(x2) + ' ' + str(y2) + ' ' + str(rl) + ' ' + str(ibs) + ' ' + str(ls))

        os.system('mv ' + MAGARRO_HOME + '/' + salida + '0001.png ' + salida + EXT)
        #os.system('mv ' + salida + '0001.png ' + salida + EXT)

	if (method != 'PT'):
	    PTimage = self.Repository.getWU(str(self.CurrentIdWork) + '_' + str(self.CurrentIdZone) + EXT)
	    array.array('B', PTimage).tofile(open(salida + 'PT' + EXT, 'w'))
	    os.system('python HistoMatch.py '+ salida + EXT + ' ' + salida + 'PT' + EXT + ' ' + salida + 'HM' + EXT)
	    salida += 'HM'

        end = time.time()

        # Update the blackboard.
        finalTime = int((end - begin))
        self.Blackboard.finishWorkUnit(self.CurrentIdWork, self.CurrentIdZone, finalTime, int(ibs), int(ls), int(rl))

        salida += EXT
        # Send the work done to the Master.
        partialImage = array.array('B', open(salida).read()).tolist()
        self.Master.giveFinalImage(self.CurrentIdWork, self.CurrentIdZone, partialImage, x1, y1,x2, y2, int(ibs))

        return finalTime

    def loadDataFuzzySystem (self, optimizationLevel):

        c = self.getCurrentFuzzyComplexity()
        nd = self.getCurrentFuzzyNeighbourDifference()
        s = self.getCurrentFuzzySize()

        print 'C: ' + str(c) + ' Nd: ' + str(nd) + ' S: ' + str(s) + ' Op: ' + str(optimizationLevel)

        inputData = []
        inputData.append((('C', c), ('Nd', nd), ('S', s), ('Op', optimizationLevel * 5)))

        for x in inputData:
            a, b, c = self.FuzzySystem.evalAll(x)
            return self.FuzzySystem.out(x)

    def loadFuzzySystem (self):

        # Get the input parameters of the fuzzy system --> Complexity, neighbour difference, size, and optimization level.
        complexity = self.getFuzzyComplexity()
        neighbourDifference = self.getFuzzyNeighbourDifference()
        size = self.getFuzzySize()

        fuzzy = UtilFuzzy()
        xmlFile = os.path.join(INITDIR, self.Name + '.xml')
        os.system('cp ' + INPUTFUZZY + ' ' + xmlFile)

        # Update of the file which specifies the fuzzy system.
        Util.createInput(self.Name + '.txt', complexity, neighbourDifference, size)
        sInput, cInput, ndInput = Util.loadInput(self.Name + '.txt')

        # Load the fuzzy system into memory.
        fuzzy.loadInputData(sInput, cInput, ndInput, xmlFile)
        fuzzy.loadFile(xmlFile)

        self.FuzzySystem = FuzzySystem(fuzzy.getSystemName())

        for x in fuzzy.getVariables():
            self.FuzzySystem.insertVar(x)

        for i in range(len(fuzzy.getAntecedents())):
            self.FuzzySystem.insertRule(fuzzy.getAntecedents()[i], fuzzy.getConsequences()[i])

    def internAdjustment (self, finalTime):

        # Get the information of the zone.
        register = self.Blackboard.read(self.CurrentIdWork, self.CurrentIdZone)
        # Get the estimated time.
        test = register.Test
        # The render will be awarded if it is done with an error margin.
        testRange = (int(test - test / RES), int(test + test / RES))
	self.RenderedZones.append(self.CurrentIdZone)   # Add this Rendered Zone
        if finalTime <= testRange[1]:
            # Prize --> Increment the credits and update the historic.
            print 'PRIZE! --> IdWork: ' + str(self.CurrentIdWork) + ' IdZone: ' + str(self.CurrentIdZone) + ' Test: ' +str(test) + ' Treal: ' + str(finalTime)
            self.Credits += PRIZE
            self.Historic.append(GOODRESULT)
            # Adjust the intern coefficient.
            if finalTime <= testRange[0]:
                self.adjustInternCoefficient(True, finalTime, testRange[0])

        else:
            # FAILURE --> Decrement the credits and update the historic.
            print 'FAILURE! --> IdWork: ' + str(self.CurrentIdWork) + ' IdZone: ' + str(self.CurrentIdZone) + ' Test: ' +str(test) + ' Treal: ' + str(finalTime)
            self.Credits += FAILURE
            self.Historic.append(BADRESULT)
            # Adjust the intern coefficient.
            self.adjustInternCoefficient(False, finalTime, testRange[1])

    def adjustInternCoefficient (self, increment, finalTime, test):

        print '***Current coefficient: ' + str(self.Coefficient)
        if increment:
            self.Coefficient = self.Coefficient + (self.Coefficient * ADJUST)
        else:
            self.Coefficient = self.Coefficient - (self.Coefficient * ADJUST)
        print '***New coefficient: ' + str(self.Coefficient)

    def getFuzzyComplexity (self):

        complexity = {}
        minComp, maxComp = INFINITO, MENOSINFINITO

        for zone in self.CurrentZones:
            if zone.m < minComp:
                minComp = zone.m
            if zone.m > maxComp:
                maxComp = zone.m

        interval = (maxComp - minComp) / (NUMPARTSCOMPLEXITY + 1)
        minComp -= 1
        maxComp += 1

        complexity['VS'] = (minComp, minComp, minComp + interval, minComp + 2 * interval)
        complexity['S'] = (minComp + interval, minComp + 2 * interval, minComp + 2 * interval, minComp + 3 * interval)
        complexity['N'] = (minComp + 2 * interval, minComp + 3 * interval, maxComp - 3 * interval, maxComp - 2 * interval)
        complexity['B'] = (maxComp - 3 * interval, maxComp - 2 * interval, maxComp - 2 * interval, maxComp - interval)
        complexity['VB'] = (maxComp - 2 * interval, maxComp - interval, maxComp, maxComp)

        return complexity

    def getCurrentFuzzyComplexity (self):
        return self.Blackboard.read(self.CurrentIdWork, self.CurrentIdZone).Comp

    def getFuzzyNeighbourDifference (self):

        zone = self.getZone(self.CurrentIdZone)
        differences = {}
        aux = []

        # Study the difference of the neighbour complexity.
        for x in self.CurrentZones:
            ac = 0
            for y in self.CurrentZones:
                if x.id != y.id and self.isNeighbour(x, y):
                    ac += self.getNeighbourDifference(x, y)
            aux.append(ac)

        aux.sort()
        minD, maxD = aux[0], aux[len(aux) - 1]
        interval = (maxD - minD) / (NUMPARTSNEIGHBOUR + 1)
        minD -= 1
        maxD += 1

        differences['VS'] = (minD, minD, minD + interval, minD + 2 * interval)
        differences['S'] = (minD + interval, minD + 2 * interval, minD + 2 * interval, minD + 3 * interval)
        differences['N'] = (minD + 2 * interval, minD + 3 * interval, maxD - 3 * interval, maxD - 2 * interval)
        differences['B'] = (maxD - 3 * interval, maxD - 2 * interval, maxD - 2 * interval, maxD - interval)
        differences['VB'] = (maxD - 2 * interval, maxD - interval, maxD, maxD)

        return differences

    def getCurrentFuzzyNeighbourDifference (self):

        # Study the difference of the neighbour complexity.
        for x in self.CurrentZones:
            ac = 0
            for y in self.CurrentZones:
                if x.id != y.id and self.isNeighbour(x, y):
                    ac += self.getNeighbourDifference(x, y)
            # Complexity difference between neighbours related to the current work.
            if x.id == self.CurrentIdZone:
                return ac

    def isNeighbour (self, zone1, zone2):

        if (zone1.x2 == zone2.x1 and zone1.y1 == zone2.y1) or (zone1.y2 == zone2.y1 and zone1.x1 == zone2.x1) or (zone2.x2 == zone1.x1 and zone2.y1 == zone1.y1) or (zone1.x1 == zone2.x1 and zone2.y2 == zone1.y1):
            return True
        else:
            return False

    def getNeighbourDifference (self, zone1, zone2):
        return math.fabs(zone1.m - zone2.m)

    def getFuzzySize (self):

        size = {}
        minComp, maxComp = INFINITO, MENOSINFINITO

        for zone in self.CurrentZones:
            s = Util.getZoneSize(zone.x1, zone.y1, zone.x2, zone.y2)
            if s < minComp:
                minComp = s
            if s > maxComp:
                maxComp = s

        interval = (maxComp - minComp) / (NUMPARTSSIZE + 1)
        minComp -= 1
        maxComp += 1

        size['S'] = (minComp, minComp, minComp + interval, minComp + 2 * interval)
        size['N'] = (minComp + interval, minComp + 2 * interval, maxComp - 2 * interval, maxComp - interval)
        size['B'] = (maxComp - 2 * interval, maxComp - interval, maxComp, maxComp)

        return size

    def getCurrentFuzzySize (self):

        reg = self.Blackboard.read(self.CurrentIdWork, self.CurrentIdZone)
        z = self.getZone(self.CurrentIdZone)
        s = Util.getZoneSize(z.x1, z.y1, z.x2, z.y2)

        return s

    def subscribeMaster (self, prx):

        self.Master.subscribe(self.Name, prx)
        print self.Name + ' --> Registrado en el master.'

    def unsubscribeMaster (self):

        self.Master.unsubscribe(self.Name)
        print self.Name + ' --> Dado de baja en el master.'

    def clear (self):

        xmlFile = os.path.join(INITDIR, self.Name + '.xml')
        os.system('rm -f ' + xmlFile)
        os.system('rm -f ' + self.Name + '.txt')
        os.system('rm -rf ' + self.Name)

class RenderAgent (Ice.Application):

    def run (self, argv):
        self.shutdownOnInterrupt()
        
        adapter = self.communicator().createObjectAdapter("AgentAdapter")
        adapter.activate()
        
        id = self.communicator().stringToIdentity(argv[1])
        agentI = RenderAgentI(argv[1])
        proxy = adapter.add(agentI, id)
        agentPrx = FIPA.RenderAgentPrx.uncheckedCast(proxy)
        print 'Starting up the RenderAgent ' + agentPrx.ice_toString()

        prx = self.communicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession(argv[1], 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.addObjectWithType(agentPrx, agentPrx.ice_id())
        except IceGrid.ObjectExistsException:
            print agentPrx.ice_toString() + 'exists.'

        prx2 = self.communicator().stringToProxy('FIPA/Query')
        queryPrx = IceGrid.QueryPrx.checkedCast(prx2)
        obj2 = queryPrx.findObjectById(self.communicator().
                                       stringToIdentity('DF/topic.DirectoryFacilitator'))
        topic = IceStorm.TopicPrx.uncheckedCast(obj2)
        
        try:
            topic.subscribeAndGetPublisher(None, proxy)
        except IceStorm.NoSuchTopic:
            print 'The topic DF/topic.DirectoryFacilitator does not exist.'

        obj4 = queryPrx.findObjectById(self.communicator().stringToIdentity('df'))
        dfPrx = FIPA.DirectoryFacilitatorPrx.uncheckedCast(obj4)
        if dfPrx:
            description = FIPA.DfAgentDescription()
            description.proxy = agentPrx
            agentDesc = FIPA.AgentDescription()
            agentDesc.leaseTime = 0
            s = []
            s.append(FIPA.ServiceDescription('RenderAgent', 'magarro'))            
            agentDesc.services = s
            description.desc = agentDesc
            dfPrx.register(description)            

        obj3 = queryPrx.findObjectById(self.communicator().stringToIdentity('acc'))
        accPrx = FIPA.AgentCommunicationChannelPrx.uncheckedCast(obj3)
        print agentPrx.ice_toString() + ' is online.'

        # SPECIFIC
        agentI.obtainProxies(self.communicator())
        # Notify the benchmark time.
        if agentI.Master:
            agentI.notifyBenchmarkValue()

        if agentI.Master:
            agentI.subscribeMaster(agentPrx)
        # END_OF_SPECIFIC

        self.communicator().waitForShutdown()

        topic.unsubscribe(proxy)
        agentI.clear()

        return 0

RenderAgent().main(sys.argv, MAGARRO_HOME + '/../../../config/config.agent')
