#!/usr/bin/python
# -*- coding: utf-8 -*-

import zipfile, os

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

import Ice

if HOST not in LOCAL_HOSTS:
	Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/../../../slice/FIPA.ice')
else:
	Ice.loadSlice('--all -I../../../slice -I/usr/share/slice ../../../slice/FIPA.ice')
import FIPA

def getFileName (namelist, ext):

    for f in namelist:
        r = f.split('.')
        if len(r) == 2 and r[1] == ext:
            return f

    return None

def createInput (file, complexity, neighbourDifference, size):

    f = open(file, 'w')

    for k in complexity.keys():
        f.write('C ' + str(k) + ' ' + str(complexity[k][0]) + ' ' + str(complexity[k][1]) + ' ' + str(complexity[k][2]) + ' ' + str(complexity[k][3]) + '\n')
    for k in neighbourDifference.keys():
        f.write('Nd ' + str(k) + ' ' + str(neighbourDifference[k][0]) + ' ' + str(neighbourDifference[k][1]) + ' ' + str(neighbourDifference[k][2]) + ' ' + str(neighbourDifference[k][3]) + '\n')
    for k in size.keys():
        f.write('S ' + str(k) + ' ' + str(size[k][0]) + ' ' + str(size[k][1]) + ' ' + str(size[k][2]) + ' ' + str(size[k][3]) + '\n')

    f.close()

def loadInput (file):

    s, c, nd = {}, {}, {}

    f = open(file)
    
    for x in f:
        y = x.split()
        if y[0] == 'C':
            c[y[1]] = (y[2], y[3], y[4], y[5])
        elif y[0] == 'S':
            s[y[1]] = (y[2], y[3], y[4], y[5])
        elif y[0] == 'Nd':
            nd[y[1]] = (y[2], y[3], y[4], y[5])

    f.close()

    return s, c, nd

def getSize (idZone, zones):

    for z in zones:
        if idZone == z.id:
            return (z.x2 - z.x1) * (z.y2 - z.y1)

    return -1

def getZoneSize(x1, y1, x2, y2):
    return (x2 - x1) * (y2 - y1)

def traduceState (state):

    if state == 0:
        return 'Initiated'
    elif state == 1:
        return 'Active'
    elif state == 2:
        return 'Suspended'
    elif state == 3:
        return 'Waiting'
    elif state == 4:
        return 'Transit'
    else:
        return ''

def traduceExplanation (explanation):

    if explanation == 1:
        return 'Duplicate'
    elif explanation == 2:
        return 'Access'
    elif explanation == 3:
        return 'Invalid'
    elif explanation == 4:
        return 'Success'
    elif explanation == 5:
        return 'NotFound'
    else:
        return ''

class Zip:

    # name represents the file path.
    def __init__ (self, name):
        
        self.Name = name + 'zip'
        self.File = zipfile.ZipFile(self.Name, 'w', zipfile.ZIP_DEFLATED)

    def getName (self):
        return self.Name

    def getFile (self):
        return self.File

    def add (self, file):
        self.getFile().write(file)

    def close (self):
        """Cierra el archivo"""
        self.getFile().close()
    
class Unzip:

    def __init__ (self, name):
        """Crea un objeto del tipo Unzip"""

        self.Name = name
        self.File = zipfile.ZipFile(self.Name, 'r')

    def getName (self):
        return self.Name

    def getFile (self):
        return self.File

    def extract (self, extractDir = './render'):

        os.chdir(extractDir)
        for x in self.getFile().namelist():
            outfile = file(x, 'w')
            outfile.write(self.getFile().read(x))
            outfile.flush()
            outfile.close()
        os.chdir('..')

        return self.getFile().namelist()
