#!/usr/bin/python
# -*- coding: utf-8 -*-

import Image
import os, os.path, PIL.Image
import array

RESX, RESY = 400, 300

def cropImage(file, ext, b1, b2, b3, b4):
    im = Image.open(file)
    box = (int(b1), int(b2), int(b3), int(b4))
    region = im.crop(box)
    region.save(file, ext)

def meanDifference(file1, file2):
    command = "convert "+ file1 +" "+ file2 +" -compose difference -composite -fx '(r+g+b)/3' miff:- |"
    command = command + " identify -verbose - | "
    command = command + " sed -n '/^.*Mean: */{s//scale=5;/;s/(.*)//;s/$/*100\/32768/;p;q;}' | bc" 
    pipe = os.popen3(command)
    return (float(pipe[1].read())/2.0)


def saveImgDifference(file1, file2, dest):
    command = "convert "+ file1 +" "+ file2 +" -compose difference -composite -fx '(r+g+b)/3' " + dest
    os.system(command)


def createMask(width, height):
        
    s=""
    
    offset = 255.0 / width
    data = []
    for i in range(0, width):
        data.append(i*offset)
        
    for i in range(0,height):
        s += array.array("I", data).tostring()

    mask = PIL.Image.fromstring('RGBA', (width,height), s)
    return mask

def createLowResolutionImage (pathToImage, fileName, finalPath):

    newFileName = fileName.split('.')[0] + '.jpg'
    image = Image.open(os.path.join(pathToImage, fileName))
    out = image.resize((RESX, RESY))
    image.save(os.path.join(finalPath, newFileName), 'JPEG')
#    out.save(os.path.join(finalPath, 'small' + newFileName), 'JPEG')
#    os.system("chmod 777 "+ os.path.join(finalPath, 'small' + newFileName))

class PieceOfImage:

    def __init__ (self, x1, y1, x2, y2, ibs):

        # Coordinates.
        self.X1, self.Y1 = x1, y1
        self.X2, self.Y2 = x2, y2
        # Resolution of the final image.
        self.ResX, self.ResY = 0, 0
        # Interpolation band size.
        self.Ibs = ibs
        # Neighours shows if an interpolation has been carried out with the neighbours.
        # Ej. Neighbours['L'] == 1 ==> A left neighbour.
        self.Neighbours = {}
        self.Neighbours['L'], self.Neighbours['R'] = 0, 0
        self.Neighbours['U'], self.Neighbours['D'] = 0, 0

    def getXSize (self):
        return self.X2 - self.X1

    def getYSize (self):
        return self.Y2 - self.Y1    

    def getUpIbs (self):
        return max(self.Y1 - self.Ibs, 0)

    def getDownIbs (self):
        return min(self.Y2 + self.Ibs, self.ResY)

    def getLeftIbs (self):
        return max(self.X1 - self.Ibs, 0)

    def getRightIbs (self):
        return min(self.X2 + self.Ibs, self.ResX)

    def isRightNeighbour (self, p):

        if self.X2 == p.X1 and (self.Y1 == p.Y1 or self.Y2 == p.Y2 or (self.Y1 < p.Y1 and self.Y2 > p.Y2) or (self.Y1 > p.Y1 and self.Y2 < p.Y2)):
            if self.getYSize() > p.getYSize():
                return True, (p.X1, p.Y1, p.X1, p.Y2)
            else:
                return True, (self.X2, self.Y1, self.X2, self.Y2)
        else:
            return False, None

    def isLeftNeighbour (self, p):

        if self.X1 == p.X2 and (self.Y1 == p.Y1 or self.Y2 == p.Y2 or (self.Y1 < p.Y1 and self.Y2 > p.Y2) or (self.Y1 > p.Y1 and self.Y2 < p.Y2)):
            if self.getYSize() > p.getYSize():
                return True, (p.X2, p.Y1, p.X2, p.Y2)
            else:
                return True, (self.X1, self.Y1, self.X1, self.Y2)
        else:
            return False, None

    def isDownNeighbour (self, p):

        if self.Y2 == p.Y1 and (self.X1 == p.X1 or self.X2 == p.X2 or (self.X1 < p.X1 and self.X2 > p.X2) or (self.X1 > p.X1 and self.X2 < p.X2)):
            if self.getXSize() > p.getXSize():
                return True, (p.X1, p.Y1, p.X2, p.Y1)
            else:
                return True, (self.X1, self.Y2, self.X2, self.Y2)
        else:
            return False, None

    def isUpNeighbour (self, p):

        if self.Y1 == p.Y2 and (self.X1 == p.X1 or self.X2 == p.X2 or (self.X1 < p.X1 and self.X2 > p.X2) or (self.X1 > p.X1 and self.X2 < p.X2)):
            if self.getXSize() > p.getXSize():
                return True, (p.X1, p.Y2, p.X2, p.Y2)
            else:
                return True, (self.X1, self.Y1, self.X2, self.Y1)
        else:
            return False, None
        
    def __eq__ (self, p):

        if self.X1 == p.X1 and self.X2 == p.X2 and self.Y1 == p.Y1 and self.Y2 == p.Y2:
            return True
        else:
            return False

    def __str__ (self):
        return '[' + str(self.X1) + ', ' + str(self.Y1) + ', ' + str(self.X2) + ', ' + str(self.Y2) + '] Ibs: ' + str(self.Ibs)

class ImageSet:

    def __init__ (self):
        self.Images = {}

    def add (self, id, pieceOfImage):
        self.Images[id] = pieceOfImage

    def getImage (self, i):
        return self.getImages.get(i)

    def setNeighbourhood (self, piece):

        r, l, u, d = 0, 0, 0, 0

        for p in self.Images.values():

            if piece != p:
                # Right neighbours.
                if piece.X2 == p.X1 and (piece.Y1 == p.Y1 or piece.Y2 == p.Y2 or (piece.Y1 < p.Y1 and piece.Y2 > p.Y2) or (piece.Y1 > p.Y1 and piece.Y2 < p.Y2)):
                    r += 1
                # Left neighbours.
                if piece.X1 == p.X2 and (piece.Y1 == p.Y1 or piece.Y2 == p.Y2 or (piece.Y1 < p.Y1 and piece.Y2 > p.Y2) or (piece.Y1 > p.Y1 and piece.Y2 < p.Y2)):
                    l += 1
                # Up neighbours.
                if piece.Y1 == p.Y2 and (piece.X1 == p.X1 or piece.X2 == p.X2 or (piece.X1 < p.X1 and piece.X2 > p.X2) or (piece.X1 > p.X1 and piece.X2 < p.X2)):
                    u += 1
                # Down neighbours.
                if piece.Y2 == p.Y1 and (piece.X1 == p.X1 or piece.X2 == p.X2 or (piece.X1 < p.X1 and piece.X2 > p.X2) or (piece.X1 > p.X1 and piece.X2 < p.X2)):
                    d += 1

        piece.Neighbours['R'] = r
        piece.Neighbours['L'] = l
        piece.Neighbours['U'] = u
        piece.Neighbours['D'] = d
