#ifndef _FIPA
#define _FIPA

#include <Ice/BuiltinSequences.ice>

module FIPA
{

  enum EState {Initiated, Active, Suspended, Waiting, Transit};

  struct ServiceDescription
  {
    string name;
    string type;
  };

  sequence<ServiceDescription> ServiceDescriptionSeq;

  struct AgentDescription
  {
    ServiceDescriptionSeq services;
    Ice::StringSeq protocols;
    Ice::StringSeq ontologies;
    Ice::StringSeq languages;
    int leaseTime;
  };

  enum ACLRepresentation {bitEfficientRep, stringRep, xmlRep};
	
  interface Agent;

  struct Payload
  {
    string performative;
    Agent* sender;
    Agent* receiver;
    string content;
    string language;
    string ontology;
    string protocol;
    int conversationId;
  };

  interface Agent
  {
    idempotent void setState(EState state);
    idempotent EState getState();
    void inform(Agent* proxy, ServiceDescriptionSeq services);
    ["ami"] void receiveMessage(ACLRepresentation encoding, Payload content);
    void destroy();
  };

  sequence<Agent*> AgentSeq;

  class AgentPersistent implements Agent
  {
    EState state;
    AgentDescription description;
  };

  struct DfAgentDescription
  {
    Agent* proxy;
    AgentDescription desc;
  };

  interface DirectoryFacilitator
  {
    void register(DfAgentDescription desc);
    void deregister(Agent* a);
  };

  sequence<DfAgentDescription> AgentDirectory;

  class DirectoryFacilitatorPersistent implements DirectoryFacilitator
  {
    AgentDirectory agents;
  };

  struct Envelope
  {
    AgentSeq to;
    Agent* xfrom;
    string date;
    ACLRepresentation rep;		
  };

  struct Message
  {
    Envelope transportInfo;
    Payload content;
  };

  interface AgentCommunicationChannel
  {
    ["ami", "cpp:const"] void receiveMessage(Message m);
  };

};

#endif