<?php 

// imagelinethick /////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

function imagelinethick($image, $x1, $y1, $x2, $y2, $color, $thick = 1)
{
    if ($thick == 1) {
        return imageline($image, $x1, $y1, $x2, $y2, $color);
    }
    $t = $thick / 2 - 0.5;
    if ($x1 == $x2 || $y1 == $y2) {
        return imagefilledrectangle($image, round(min($x1, $x2) - $t), 
               round(min($y1, $y2) - $t), round(max($x1, $x2) + $t), round(max($y1, $y2) + $t), $color);
    }
    $k = ($y2 - $y1) / ($x2 - $x1); //y = kx + q
    $a = $t / sqrt(1 + pow($k, 2));
    $points = array(
        round($x1 - (1+$k)*$a), round($y1 + (1-$k)*$a),
        round($x1 - (1-$k)*$a), round($y1 - (1+$k)*$a),
        round($x2 + (1+$k)*$a), round($y2 - (1-$k)*$a),
        round($x2 + (1-$k)*$a), round($y2 + (1+$k)*$a),
    );
    imagefilledpolygon($image, $points, 4, $color);
    return imagepolygon($image, $points, 4, $color);
}

// drawprogbar ////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

function drawprogbar($percent, $isanalysis)
{
  echo '<table style="width:100%; height:23px;" border="0" cellpadding="0" cellspacing="0"><tbody><tr>';
  if ($percent == 100 && $isanalysis == false) {
    echo '<td style="width:200px;background-color:#eae9ef;text-align:right;"><span class="base">';
    echo '<a href="output.jpg" target="_blank">[Click Here to Download Final Image]</a>&nbsp;&nbsp;</span></td>';
  } 
  else {
    echo '   <td style="width:'. $percent .'%; text-align: left; ';
    echo '              background-image:url(\'images/progbarcenter-on.jpg\');">';
    echo '       <img width="2" src="images/progbarleft.jpg">';
    echo '   </td>';
    echo '   <td style="text-align: left; background-image:url(\'images/progbarcenter-off.jpg\');">';
    echo '       <img width="2" src="images/progbarright-on.jpg">';
    echo '   </td>';
    echo '   <td style="width: 2px;"><img src="images/progbarright-off.jpg"></td>'; 
  }
  echo '   </tr></tbody></table>';
}

// ImageRectangleWithRoundedCorners ///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

function imagerectanglewithroundedcorners(&$im, $x1, $y1, $x2, $y2, $radius, $color) {
  // draw rectangle without corners
  imagefilledrectangle($im, $x1+$radius, $y1, $x2-$radius, $y2, $color);
  imagefilledrectangle($im, $x1, $y1+$radius, $x2, $y2-$radius, $color);
  // draw circled corners
  imagefilledellipse($im, $x1+$radius, $y1+$radius, $radius*2, $radius*2, $color);
  imagefilledellipse($im, $x2-$radius, $y1+$radius, $radius*2, $radius*2, $color);
  imagefilledellipse($im, $x1+$radius, $y2-$radius, $radius*2, $radius*2, $color);
  imagefilledellipse($im, $x2-$radius, $y2-$radius, $radius*2, $radius*2, $color);
}

// imagecenteredstring ////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

function imagecenteredstring($image, $font_size, $posx, $posy, $text, $text_color){
  $colorfill = imagecolorallocatealpha ($image, 0, 0, 0, 0);

  $initx = round($posx-((strlen($text)*imagefontwidth($font_size))/2), 1);
  $inity = round($posy-(imagefontheight($font_size)/2));

  imagerectanglewithroundedcorners ($image, $initx - 3, $inity - 3, 
				    $initx + 3 + round(strlen($text)*imagefontwidth($font_size),1), 
				    $inity + 9 + round(imagefontheight($font_size)/2), 5, $colorfill);

  imagestring($image, $font_size, $initx, $inity, $text, $text_color);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// main ///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

$partition = (int) $_GET['part'];
$draw_partition = (bool) $_GET['dp'];
$draw_prog_partition = (bool) $_GET['dpp'];
$img = (int) $_GET['img'];

Ice_loadProfile("Profile2");
try {
  $analystPrx = $ICE->stringToProxy("analyst")->ice_checkedCast("::FIPA::Analyst");
  $listofzones = $analystPrx->getZonesByPartition($partition);
  $masterPrx = $ICE->stringToProxy("master")->ice_checkedCast("::FIPA::Master");				
  $lregister = $masterPrx->getBlackboardState();
}
catch (Ice_LocalException $ex) {
  print_r($ex);
}

list($width, $height) = getimagesize('output.jpg');
$newwidth = 600;
$newheight = 450;

$fw = $newwidth / ($width * 1.0);
$fh = $newheight / ($height * 1.0);

$thumb = imagecreatetruecolor($newwidth, $newheight);
$source = imagecreatefromjpeg('output.jpg');
imagecopyresized($thumb, $source, 0, 0, 0, 0, $newwidth, $newheight, $width, $height);

$colorline = imagecolorallocatealpha ($thumb, 255, 30, 30, 0);
$colorfill = imagecolorallocatealpha ($thumb, 30, 255, 30, 80);
$colortext = imagecolorallocate($thumb, 255, 255, 255);

$nzonesdone = 0;
$nzones = count($listofzones);
foreach ($listofzones as $zone) {
  $time = 0;
  foreach ($lregister as $reg) {
    if ($reg->WorkUnit == $zone->id) {
      $time = $reg->Test;
      if (($time != 0) && ($draw_prog_partition == true)) $nzonesdone++;
      if (($reg->State == 0) && ($draw_prog_partition == false)) $nzonesdone++;
      break;
    }
  }
  
  if ($time != 0 && $draw_prog_partition) {
    imagefilledrectangle ($thumb, $zone->x1 * $fw, $zone->y1 * $fh, $zone->x2 * $fw, $zone->y2 * $fh, $colorfill);
    imagecenteredstring($thumb, 9, ($zone->x2 - (($zone->x2 - $zone->x1)/2.0))*$fw, 
			($zone->y2 - (($zone->y2 - $zone->y1)/2.0))*$fh, $zone->id .":". $time, $colortext);
  }

  if (!$draw_prog_partition && $draw_partition) {
    imagecenteredstring($thumb, 9, ($zone->x2 - (($zone->x2 - $zone->x1)/2.0))*$fw, 
			($zone->y2 - (($zone->y2 - $zone->y1)/2.0))*$fh, $zone->id, $colortext);
  }

  if ($draw_partition) {
    imagelinethick($thumb, $zone->x1 * $fw, $zone->y1 * $fh, $zone->x2 * $fw, $zone->y1 * $fh, $colorline, 3);
    imagelinethick($thumb, $zone->x2 * $fw, $zone->y1 * $fh, $zone->x2 * $fw, $zone->y2 * $fh, $colorline, 3);
    imagelinethick($thumb, $zone->x1 * $fw, $zone->y1 * $fh, $zone->x1 * $fw, $zone->y2 * $fh, $colorline, 3);
    imagelinethick($thumb, $zone->x1 * $fw, $zone->y2 * $fh, $zone->x2 * $fw, $zone->y2 * $fh, $colorline, 3);
  }
}

// Version original: imagejpeg($thumb, 'smalloutput.jpg', 90);

if ($nzonesdone == 0) $percent = 1;
else $percent = ($nzonesdone / ($nzones * 1.0)) * 100;


if ($img == 1) {  // Update the image
  header('Content-type: image/jpg');
  imagejpeg($thumb,"", 70);
} else {  // Update the progress bar
  drawprogbar($percent, $draw_prog_partition);
}

imagedestroy($source);
imagedestroy($thumb);

?>
