#include <Ice/Application.h>
#include <IceGrid/Admin.h>
#include <IceGrid/Query.h>
#include <IceGrid/Registry.h>
#include <IceUtil/IceUtil.h>
#include <IceStorm/IceStorm.h>
#include <Freeze/Freeze.h>
#include <FIPA.h>
#include <AgentI.h>
#include <signal.h>

using namespace std;

class Agent : virtual public Ice::Application {

private:
  FIPA::AgentPrx agentPrx;

public:
  
  virtual int run (int argc,
		   char* argv[])
  {

    Ice::ObjectAdapterPtr adapter;
    Ice::ObjectPrx prx1, prx2, prx3;
    AgentIPtr agentI;
    FIPA::AgentPrx agentPrx2;
    IceGrid::AdminPrx adminPrx;
    IceGrid::QueryPrx queryPrx;
    Ice::Identity id;
    FIPA::AgentDescription ad;
    FIPA::DirectoryFacilitatorPrx dfPrx;
    FIPA::AgentCommunicationChannelPrx accPrx;
    IceStorm::TopicPrx topic;
    Ice::ObjectPrx prx;

    communicator()->addObjectFactory(new AgentFactory(),
				     FIPA::AgentPersistent::ice_staticId());
    adapter = communicator()->createObjectAdapter("AgentAdapter");

    AgentI::_evictor = Freeze::createEvictor(adapter, "db", "agent", new AgentInitializer());    
    adapter->activate();
    adapter->addServantLocator(AgentI::_evictor, "");

    for (int i = 0;
	 i < 1;
	 i++)
      {
	id = communicator()->stringToIdentity(IceUtil::generateUUID());
	agentI = new AgentI(ad);
	prx3 = AgentI::_evictor->add(agentI, id);
	agentPrx = FIPA::AgentPrx:: uncheckedCast(prx3);
	cout << "I am " << communicator()->identityToString(agentPrx->ice_getIdentity()) 
	     << endl;
	
	prx1 = communicator()->stringToProxy("FIPA/Registry");
	IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx1);
	IceGrid::AdminSessionPrx adminSessionPrx;
	try
	  {
	    adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
	    adminPrx = adminSessionPrx->getAdmin();       
	  }
	catch (const IceGrid::PermissionDeniedException& ex)
	  {
	    cout << "permission denied:\n" << ex.reason << endl;
	  }
	catch (const IceUtil::NullHandleException& ex)
	  {
	    cout << "null handle exception:\n" << endl;
	  }

	try
	  {
	    adminPrx->addObjectWithType(agentPrx, agentPrx->ice_id());
	  }
	catch (const IceGrid::ObjectExistsException&)
	  {
	  }
	catch (const IceUtil::NullHandleException& ex)
	  {
	    cout << "null handle exception:\n" << endl;
	  }

	/*** ICESTORM ***/

	prx = adapter->getCommunicator()->stringToProxy("FIPA/Query");
	queryPrx = IceGrid::QueryPrx::checkedCast(prx);

	topic = IceStorm::TopicPrx::uncheckedCast(queryPrx->findObjectById(adapter->getCommunicator()->stringToIdentity("DF/topic.DirectoryFacilitator")));
	try
	  {
	    IceStorm::QoS qos;
	    topic->subscribeAndGetPublisher(qos, prx3);
	  }
	catch (const IceStorm::NoSuchTopic&)
	  {
	  }

	/*** ICESTORM ***/
	
	accPrx = FIPA::AgentCommunicationChannelPrx::
	  uncheckedCast(queryPrx->findObjectById(communicator()->stringToIdentity("acc")));
	cout << accPrx->ice_toString() << endl;
	FIPA::Message m;
	m.content.protocol = "fipa-request";
	m.content.performative = "request";
	m.content.content = "request";
	m.content.conversationId = 5694908;
	if (argc >= 2)
	  {
	    FIPA::AgentPrx p = FIPA::AgentPrx::
	      uncheckedCast(communicator()->stringToProxy(argv[1]));
	    m.content.sender = agentPrx;
	    m.content.receiver = p;
	    m.transportInfo.to.push_back(p);
	    m.transportInfo.xfrom = agentPrx;
	    m.transportInfo.rep = FIPA::stringRep;
	    if (argc == 3)
	      {
		FIPA::AgentPrx p2 = FIPA::AgentPrx::
		  uncheckedCast(communicator()->stringToProxy(argv[2]));
		m.transportInfo.to.push_back(p2);
	      }
	    cout << "Trying to send a message." << endl;
	    agentI->sendMessage(m, accPrx);
	  }
		
	dfPrx = FIPA::DirectoryFacilitatorPrx::
	  uncheckedCast(queryPrx->findObjectById(communicator()->stringToIdentity("df")));
	if (dfPrx)
	  {
	    FIPA::DfAgentDescription description;
	    description.proxy = agentPrx;
	    FIPA::AgentDescription adesc;
	    FIPA::ServiceDescription sd;
	    sd.name = "test";
	    sd.type = "test";
	    adesc.services.push_back(sd);
	    description.desc = adesc;
	    dfPrx->_cpp_register(description);
	    dfPrx->deregister(agentPrx);
	  }
      }

    communicator()->waitForShutdown();

    return EXIT_SUCCESS;

  }
};

int main (int argc,
	  char *argv[])
{
  Agent agent;
  return agent.main(argc, argv, "../../config/config.agent");
}
