#include <Ice/Application.h>
#include <IceGrid/Admin.h>
#include <IceGrid/Registry.h>
#include <IceBox/IceBox.h>
#include <FIPA.h>
#include <AgentCommunicationChannel.h>
#include <AgentCommunicationChannelI.h>

using namespace std;

extern "C"
{
  IceBox::Service*
  create(Ice::CommunicatorPtr communicator)
  {
    return new AgentCommunicationChannel;
  }
}

void
AgentCommunicationChannel::start(const std::string& name,
				 const Ice::CommunicatorPtr& communicator,
				 const Ice::StringSeq& args)
{
  Ice::ObjectPrx prx1;
  FIPA::AgentCommunicationChannelPrx accPrx;
  IceGrid::AdminPrx adminPrx;
  
  _adapter = communicator->createObjectAdapter("AgentCommunicationChannelAdapter");
  accPrx = FIPA::AgentCommunicationChannelPrx::
    uncheckedCast(_adapter->add(new AgentCommunicationChannelI,
				communicator->stringToIdentity("acc")));
  _adapter->activate();

  prx1 = communicator->stringToProxy("FIPA/Registry");
  IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx1);
  IceGrid::AdminSessionPrx adminSessionPrx;
  try
    {
      adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
    }
  catch (const IceGrid::PermissionDeniedException& ex)
    {
      cout << "permission denied:\n" << ex.reason << endl;
    }
  adminPrx = adminSessionPrx->getAdmin();    

  try
    {
      adminPrx->addObjectWithType(accPrx, accPrx->ice_id());
    }
  catch (const IceGrid::ObjectExistsException&)
    {
      cout << "ObjectExistsException." << endl;
    }
}

void
AgentCommunicationChannel::stop()
{
  Ice::ObjectPrx prx = _adapter->getCommunicator()->stringToProxy("FIPA/Registry");
  IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx);
  IceGrid::AdminSessionPrx adminSessionPrx;
  IceGrid::AdminPrx adminPrx;

  try
    {
      adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
    }
  catch (const IceGrid::PermissionDeniedException& ex)
    {
      cout << "permission denied:\n" << ex.reason << endl;
    }
  adminPrx = adminSessionPrx->getAdmin();    

  try
    {
      adminPrx->removeObject(_adapter->getCommunicator()->stringToIdentity("acc"));
    }
  catch (const IceGrid::ObjectNotRegisteredException&)
    {      
    }
  catch (const IceGrid::DeploymentException&)
    {
    }
  _adapter->deactivate();
}
