#include <AgentI.h>
#include <IceGrid/Admin.h>
#include <IceGrid/Query.h>
#include <IceGrid/Registry.h>
#include <IceStorm/IceStorm.h>
#include <Ice/Ice.h>

using namespace std;
using namespace FIPA;

AgentCommunicationChannel_receiveMessageI::
AgentCommunicationChannel_receiveMessageI()
{
}

void
AgentCommunicationChannel_receiveMessageI::ice_response()
{
  cout << "Message delivered." << endl;
}

void
AgentCommunicationChannel_receiveMessageI::ice_exception(const Ice::Exception& ex)
{
  cout << "Message no delivered." << endl;
}

Freeze::EvictorPtr AgentI::_evictor = NULL;

AgentI::AgentI()
{
  state = Initiated;
  _destroyed = false;
  description.leaseTime = 0;
}

AgentI::AgentI(const FIPA::AgentDescription& ad)
{
  state =Initiated;
  description = ad;
  _destroyed = false;
}

void
AgentI::setState(::FIPA::EState st,
		 const ::Ice::Current&)
{
  IceUtil::Mutex::Lock lock(*this);
  if (_destroyed)
    {
      throw Ice::ObjectNotExistException(__FILE__, __LINE__); 
    }
  state = st;
}

::FIPA::EState
AgentI::getState(const ::Ice::Current&)
{
  IceUtil::Mutex::Lock lock(*this);
  if (_destroyed)
    {
      throw Ice::ObjectNotExistException(__FILE__, __LINE__); 
    }
  return state;
}

void
AgentI::inform(const ::FIPA::AgentPrx& proxy,
	       const ::FIPA::ServiceDescriptionSeq& services,
	       const ::Ice::Current&)
{
  cout << proxy->ice_toString()
       << "\nRegistered several services: "
       << endl;
  ServiceDescriptionSeq::const_iterator it;
  for (it = services.begin();
       it != services.end();
       ++it)
    {
      cout << "\t*** Service name: " << it->name << "\n"
	   << "\t*** Service type: " << it->type << "\n"
	   << endl;
    }
}

void
AgentI::receiveMessage(::FIPA::ACLRepresentation encoding,
		       const ::FIPA::Payload& content,
		       const ::Ice::Current& curr)
{
  IceUtil::Mutex::Lock lock(*this);
  processMessage(encoding, content);
}

void
AgentI::destroy(const ::Ice::Current& curr)
{
  IceUtil::Mutex::Lock lock(*this);
  if (_destroyed)
    {
      throw Ice::ObjectNotExistException(__FILE__, __LINE__);
    }
  _destroyed = true;
  _evictor->remove(curr.id);
  Ice::ObjectPrx prx = curr.adapter->getCommunicator()->stringToProxy("FIPA/Registry");
  IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx);
  IceGrid::AdminSessionPrx adminSessionPrx;
  IceGrid::AdminPrx adminPrx;
  try
    {
      adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
    }
  catch (const IceGrid::PermissionDeniedException& ex)
    {
      cout << "permission denied:\n" << ex.reason << endl;
    }
  adminPrx = adminSessionPrx->getAdmin();       

  try
    {
      adminPrx->removeObject(curr.id);
      cout << curr.id.name << " terminated." << endl;
    }
  catch (const IceGrid::ObjectNotRegisteredException&)
    {
    }
}

void
AgentI::sendMessage(const FIPA::Message& m,
		    const FIPA::AgentCommunicationChannelPrx& acc)
{
  IceUtil::Mutex::Lock lock(*this);

  AgentCommunicationChannel_receiveMessageIPtr p =
    new AgentCommunicationChannel_receiveMessageI();
  acc->receiveMessage_async(p, m);
}

void
AgentI::processMessage(const FIPA::ACLRepresentation aclRep,
		       const FIPA::Payload& p)
{
  cout << "Processing the message..." << endl;
}

Ice::ObjectPtr
AgentFactory::create(const std::string& type)
{
  return new AgentI();
}

void
AgentFactory::destroy()
{
}

void
AgentInitializer::initialize(const Ice::ObjectAdapterPtr& oa, 
			     const Ice::Identity& ident, 
			     const std::string& str, 
			     const Ice::ObjectPtr& obj)
{
  AgentIPtr ptr = AgentIPtr::dynamicCast(obj);
}
