#ifndef AgentI_H
#define AgentI_H

#include <FIPA.h>
#include <Freeze/Freeze.h>
#include <IceUtil/IceUtil.h>
#include <map>
#include <stack>
#include <UtilXML.h>

class AgentCommunicationChannel_receiveMessageI: 
public FIPA::AMI_AgentCommunicationChannel_receiveMessage
{
 public:
  
  AgentCommunicationChannel_receiveMessageI();

  virtual void ice_response();
  virtual void ice_exception(const Ice::Exception&);
};

typedef IceUtil::Handle<AgentCommunicationChannel_receiveMessageI> AgentCommunicationChannel_receiveMessageIPtr;

enum CommunicativeAct {request, agree, refuse, inform, failure};

class AgentI: public FIPA::AgentPersistent,
	      public IceUtil::AbstractMutexI<IceUtil::Mutex>
{
  friend class AgentInitializer;
 public:
  
  AgentI();
  AgentI(const FIPA::AgentDescription&);
  
  virtual void setState(::FIPA::EState st,
			const ::Ice::Current&);
  virtual ::FIPA::EState getState(const ::Ice::Current&);
  virtual void inform(const ::FIPA::AgentPrx&,
		      const ::FIPA::ServiceDescriptionSeq&,
		      const ::Ice::Current&);
  virtual void receiveMessage(::FIPA::ACLRepresentation encoding,
			      const ::FIPA::Payload& content,
			      const ::Ice::Current&);
  virtual void destroy(const ::Ice::Current&);

  virtual void sendMessage(const FIPA::Message&, const FIPA::AgentCommunicationChannelPrx&);
  virtual void processMessage(const FIPA::ACLRepresentation, const FIPA::Payload&);

  static Freeze::EvictorPtr _evictor;

 protected:

  bool _destroyed;
};

typedef IceUtil::Handle<AgentI> AgentIPtr;

class AgentFactory: public Ice::ObjectFactory
{
 public:
  
  virtual Ice::ObjectPtr create(const std::string& type);
  virtual void destroy();
};

class AgentInitializer: public Freeze::ServantInitializer
{
 public:

  virtual void initialize(const Ice::ObjectAdapterPtr& oa, 
			  const Ice::Identity& ident, 
			  const std::string& str, 
			  const Ice::ObjectPtr& obj);
};

#endif
