#include <IceGrid/Admin.h>
#include <IceGrid/Registry.h>
#include <IceStorm/IceStorm.h>
#include <IceBox/IceBox.h>
#include <IceUtil/IceUtil.h>
#include <Freeze/Freeze.h>
#include <FIPA.h>
#include <DirectoryFacilitator.h>
#include <DirectoryFacilitatorI.h>

using namespace std;

extern "C"
{
  IceBox::Service*
  create(Ice::CommunicatorPtr communicator)
  {
    return new DirectoryFacilitator;
  }
}

void
DirectoryFacilitator::start(const std::string& name,
			    const Ice::CommunicatorPtr& communicator,
			    const Ice::StringSeq& args)
{
  Ice::ObjectPrx prx1;
  FIPA::DirectoryFacilitatorPrx dfPrx;
  IceGrid::AdminPrx adminPrx;
  Ice::Identity id;

  communicator->addObjectFactory(new DirectoryFacilitatorFactory(),
				 FIPA::DirectoryFacilitatorPersistent::ice_staticId());
  _adapter = communicator->createObjectAdapter("DirectoryFacilitatorAdapter");

  DirectoryFacilitatorI::_evictor = 
    Freeze::createEvictor(_adapter, "db", "df", new DirectoryFacilitatorInitializer());    
  _adapter->activate();
  _adapter->addServantLocator(DirectoryFacilitatorI::_evictor, "");

  id = communicator->stringToIdentity("df");

  Ice::PropertiesPtr properties = communicator->getProperties();

  const string proxyProperty = "IceStorm.TopicManager.Proxy";
  string proxy = properties->getProperty(proxyProperty);
  if(proxy.empty())
    {
      cerr << "Property `" << proxyProperty << "' not set" << endl;
      return ;
    }

  Ice::ObjectPrx base = communicator->stringToProxy(proxy);
  IceStorm::TopicManagerPrx manager = IceStorm::TopicManagerPrx::checkedCast(base);
  if(!manager)
    {
      cerr << "Invalid proxy" << endl;
      return ;
    }
  
  IceStorm::TopicPrx topic;
  try
    {
      topic = manager->retrieve("DirectoryFacilitator");
    }
  catch(const IceStorm::NoSuchTopic& e)
    {
      topic = manager->create("DirectoryFacilitator");
    }

  prx1 = communicator->stringToProxy("FIPA/Registry");
  IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx1);
  IceGrid::AdminSessionPrx adminSessionPrx;
  try
    {
      adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
    }
  catch (const IceGrid::PermissionDeniedException& ex)
    {
      cout << "permission denied:\n" << ex.reason << endl;
    }
  adminPrx = adminSessionPrx->getAdmin();    

  try
    {
      adminPrx->addObjectWithType(topic, topic->ice_id());
    }
  catch (const IceGrid::ObjectExistsException&)
    {
    }
  catch (const IceGrid::DeploymentException& ex)
    {
      cout << "DeploymentException " << ex.reason << endl;
    }
  catch (const IceUtil::NullHandleException&)
    {
      cout << "NullHandleException" << endl;
    }
  
  if (DirectoryFacilitatorI::_evictor->hasObject(id))
    {
      cout << id.name << " was in the evictor." << endl;
    }
  else
    {
      dfPrx = FIPA::DirectoryFacilitatorPrx::
	uncheckedCast(DirectoryFacilitatorI::_evictor->add(new DirectoryFacilitatorI, id));
      cout << "I am " << communicator->identityToString(dfPrx->ice_getIdentity()) << endl;
      
      try
	{
	  adminPrx->addObjectWithType(dfPrx, dfPrx->ice_id());
	}
      catch (const IceGrid::ObjectExistsException&)
	{
	  cout << "ObjectExistsException." << endl;
	}
    }
}

void
DirectoryFacilitator::stop()
{
  Ice::ObjectPrx prx = _adapter->getCommunicator()->stringToProxy("FIPA/Registry");
  IceGrid::AdminPrx adminPrx = IceGrid::AdminPrx::checkedCast(prx);
  IceGrid::RegistryPrx registryPrx = IceGrid::RegistryPrx::checkedCast(prx);
  IceGrid::AdminSessionPrx adminSessionPrx;
  try
    {
      adminSessionPrx = registryPrx->createAdminSession("david", "dummy");
    }
  catch (const IceGrid::PermissionDeniedException& ex)
    {
      cout << "permission denied:\n" << ex.reason << endl;
    }
  adminPrx = adminSessionPrx->getAdmin();    

  try
    {
      adminPrx->removeObject(_adapter->getCommunicator()->stringToIdentity("df"));
    }
  catch (const IceGrid::ObjectNotRegisteredException&)
    {      
    }
  catch (const IceGrid::DeploymentException&)
    {
    }
  _adapter->deactivate();
}
