#include <Ice/Application.h>
#include <IceUtil/Time.h>
#include <IceGrid/Admin.h>
#include <IceGrid/Query.h>
#include <Video.h>
#include <ProcessAgentI.h>

using namespace std;

class ProcessAgent : virtual public Ice::Application {
  virtual int run (int argc,
		   char* argv[])
  {

    Ice::ObjectAdapterPtr adapter = 
      communicator()->createObjectAdapter("AgentAdapter");
    Video::ProcessAgentPrx processAgent = Video::ProcessAgentPrx::checkedCast(adapter->addWithUUID(new ProcessAgentI));
    adapter->activate();
    Ice::ObjectPrx proxy = communicator()->stringToProxy("FIPA/Admin");
    IceGrid::AdminPrx admin = IceGrid::AdminPrx::checkedCast(proxy);
    Ice::ObjectPrx proxy2 = communicator()->stringToProxy("FIPA/Query");
    IceGrid::QueryPrx query = IceGrid::QueryPrx::checkedCast(proxy2);

    try
      {
	admin->addObjectWithType(processAgent, processAgent->ice_id());
      }
    catch (const IceGrid::ObjectExistsException&)
      {
	admin->updateObject(processAgent);
      }

    Video::VideoAgentPrx videoAgent = 
      Video::VideoAgentPrx::uncheckedCast(query->findObjectByType("::Video::VideoAgent"));

    if (videoAgent)
      {
	cout << videoAgent->ice_toString() << " is a ::Video::VideoAgent" << endl;

	if (argc < 2) {
	  cout << "Synopsis: ./Client <file_to_transfer>" << endl;
	  return EXIT_FAILURE;
	}
	
	string name = argv[1];
	FILE* fp = fopen(name.c_str(), "wb");
	int len = 1000 * 1024;
	int rc = EXIT_SUCCESS;
	
	try {
	  int num = 1000 * 1024;
	  Video::FilePrx file = videoAgent->read(name, num);
	  
	  int offset = 0;
	  IceUtil::Time start = IceUtil::Time::now();
	  File_nextIPtr curr, next;
	  Ice::ByteSeq bytes;
	  
	  for (;;) {
	    if (!curr) {
	      curr = new File_nextI;
	      next = new File_nextI;
	      file->next_async(curr);
	    }
	    else {
	      swap(curr, next);
	    }
	    file->next_async(next);
	    curr->getData(bytes);
	    if (bytes.empty()) {
	      break;
	    }
	    if (fwrite(&bytes[0], 1, bytes.size(), fp)
		!= bytes.size()) {
	      cerr << "error writing: " << strerror(errno) << endl;
	      rc = EXIT_FAILURE;
	    }
	    offset += bytes.size(); 
	  }
	  
	  IceUtil::Time tm = IceUtil::Time::now() - start;
	  cout << "Time employed: " << tm.toSecondsDouble() << endl;
	}
	catch (const Video::FileAccessException& e) {
	  cerr << "FileAccessException: " << e.reason << endl;
	  rc = EXIT_FAILURE;
	}
	
	fclose(fp);
      }
    
    processAgent->destroy();
    
    communicator()->waitForShutdown();
    
    return 0;

  }
};

int main (int argc,
	  char *argv[])
{
  ProcessAgent processAgent;
  return processAgent.main(argc, argv, "../../config/config.agent");
}
