#include <UtilXML.h>

using namespace std;

void
UtilXML::createProtocol(const std::string& file,
			const std::string& protocol,
			VStack& outStack)
{
  try
    {
      XMLPlatformUtils::Initialize();
    }  
  catch(const XMLException& toCatch)
    {
      char *pMsg = XMLString::transcode(toCatch.getMessage());
      XERCES_STD_QUALIFIER cerr << "Initializating error xerces-c:\n" << pMsg << endl;
      XMLString::release(&pMsg);
      return;
    }// Fin catch
  
  DOMBuilder *parser;

  try
    {

      // Objetos necesarios para el parser.
      DOMImplementation* impl =  DOMImplementationRegistry::getDOMImplementation(X("core"));
      parser = impl->createDOMBuilder(DOMImplementation::MODE_SYNCHRONOUS, 0);
            
      // Parseado.
      DOMDocument *doc;
      doc = parser->parseURI(file.c_str());

      if (doc)
	{
	  DOMElement *raiz = doc->getDocumentElement();	 
	  DOMNodeList *ips = raiz->getElementsByTagName(X("protocol"));
	  VStack stackProtocol;

	  for (int i = 0; i < ips->getLength(); ++i)
	    {
	      DOMNode *item = ips->item(i);
	      DOMNamedNodeMap *attributes = item->getAttributes();
	      DOMNode *att = attributes->getNamedItem(X("name"));
	      
	      if (att)
		{
		  const XMLCh *buf1 = att->getNodeValue();
		  char *c1 = XMLString::transcode(buf1);
		  string st1(c1);
		  if (st1 == protocol)
		    {		      
		      SVector slot;
		      // Protocol understood.
		      cout << protocol << " was recognized!" << endl;
		      DOMNodeList *interactions = item->getChildNodes();
		      for (int j = 0; j < interactions->getLength(); ++j)
			{
			  // Get the initiator/participant.
			  if (interactions->item(j)->getNodeType() == DOMNode::ELEMENT_NODE)
			    {
			      
			      const XMLCh *buf2 = interactions->item(j)->getNodeName();
			      char *c2 = XMLString::transcode(buf2);
			      string st2(c2);
			      slot.push_back(st2);

			      // Get the initiator/participant acts.			      
			      DOMNodeList *acts = interactions->item(j)->getChildNodes();
			      for (int k = 0; k < acts->getLength(); ++k)
				{
				  if (acts->item(k)->getNodeType() == DOMNode::ELEMENT_NODE)
				    {
				      const XMLCh *buf4 = acts->item(k)->
					getFirstChild()->getNodeValue();
				      char *c4 = XMLString::transcode(buf4);
				      string st4(c4);
				      slot.push_back(st4);
				    }
				}

			      stackProtocol.push(slot);
			      slot.erase(slot.begin(), slot.end());
			    }
			}
		      break;
		    } // End protocol understood.

		}

	    } // End for

	  // Reverse the stack.
	  for (; stackProtocol.size ()> 0; stackProtocol.pop())
	    {
	      outStack.push(stackProtocol.top());
	    }
	  
	}
      
    }// Fin try
  
  catch (const OutOfMemoryException&)
    {
      XERCES_STD_QUALIFIER cerr << "Insuficcient memory." << XERCES_STD_QUALIFIER endl;
      return;
    }// Fin catch
  catch (const XMLException& e)
    {
      XERCES_STD_QUALIFIER cerr << "Parser error\n:"
				<< e.getMessage() << XERCES_STD_QUALIFIER endl;
      return;
    }// Fin catch
  
  catch (const DOMException& e)
    {
      const unsigned int maxChars = 2047;
      XMLCh errText[maxChars + 1];
      
      XERCES_STD_QUALIFIER cerr << "Parser error:" << e.code << XERCES_STD_QUALIFIER endl;
      
      if (DOMImplementation::loadDOMExceptionMsg(e.code, errText, maxChars))
	XERCES_STD_QUALIFIER cerr << "Message: " << errText << XERCES_STD_QUALIFIER endl;
      
      return;
    }// Fin catch

  
  catch (...)
    {
      XERCES_STD_QUALIFIER cerr << "Parser error:\n" << XERCES_STD_QUALIFIER endl;
      return;
    }// Fin catch
  

  delete parser;
  XMLPlatformUtils::Terminate();

}

XStr::
XStr(const char* const toTranscode)
{
  fUnicodeForm = XMLString::transcode(toTranscode);
}

XStr::
~XStr()
{
  XMLString::release(&fUnicodeForm);
}

const XMLCh*
XStr::
unicodeForm() const
{
  return fUnicodeForm;
}
