#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, thread, os

HOST = os.environ['HOSTNAME']
LOCAL_HOSTS = ['exeter66', 'kurt', 'TheSecondWorldHeaven']
MAGARRO_HOME = os.environ['MAGARRO_HOME']

if HOST not in LOCAL_HOSTS:
	sys.path.append('/usr/local/IcePy-3.2.0/python')

import Ice, IceGrid

if HOST not in LOCAL_HOSTS:
	Ice.loadSlice('--all -I/usr/local/Ice-3.2.0/slice -I' + MAGARRO_HOME + '/../../../slice -I/usr/share/slice ' + MAGARRO_HOME + '/../../../slice/FIPA.ice')
else:
	Ice.loadSlice('--all -I../../../slice -I/usr/share/slice ../../../slice/FIPA.ice')
import FIPA
import UtilXML
from UtilXML import *

class AMI_Model_receiveMessageI(object):

    def ice_response(self):
        print 'Message delivered.'

    def ice_exception(self, ex):
        try:
            raise ex
        except Ice.LocalException:
            print 'Message not delivered.'

class AgentI(FIPA.Agent):

    def __init__(self, agentDescription):
        self._state = FIPA.EState.Initiated
        self._description = agentDescription
        self._destroyed = False
        self._mutex = thread.allocate_lock()

    def setState(self, state, curr):
        self._mutex.acquire()
        if self._destroyed:
            raise Ice.ObjectNotExistException
        self._state = state
        self._mutex.release()
        
    def getState(self, curr):
        self._mutex.acquire()
        if self._destroyed:
            raise Ice.ObjectNotExistException
        self._mutex.release()
        return self._state

    def inform(self, proxy, services, curr):
        if self._destroyed:
            raise Ice.ObjectNotExistException
        
        print proxy.ice_toString() + '\nRegistered several services: '
        for s in services:
            print '\t*** Service name: ' + s.name
            print '\t*** Service type: ' + s.type

    def receiveMessage(self, encoding, content, curr):
        if self._destroyed:
            raise Ice.ObjectNotExistException

        self.processMessage(encoding, content, curr)

    def destroy(self, curr):
        self._mutex.acquire()
        if self._destroyed:
            raise Ice.ObjectNotExistException
        self._destroyed = True
        
        prx = curr.adapter.getCommunicator().stringToProxy('FIPA/Registry')
        registryPrx = IceGrid.RegistryPrx.checkedCast(prx)

        try:
            adminSessionPrx = registryPrx.createAdminSession('david', 'dummy')
            adminPrx = adminSessionPrx.getAdmin()
        except IceGrid.PermissionDeniedException:
            print 'Permission denied.'
        except IceUtil.NullHandleException:
            print 'NullHandleException.'

        try:
            adminPrx.removeObject(curr.id)
            print curr.id.name + ' terminated.'
        except IceGrid.ObjectNotRegisteredException:
            print curr.id.name + ' was not registered in the registry.'

        curr.adapter.getCommunicator().shutdown()

        self._mutex.release()

    def sendMessage(self, message, accPrx):
        if self._destroyed:
            raise Ice.ObjectNotExistException

        p = AMI_Model_receiveMessageI()
        accPrx.receiveMessage_async(p, message)

    def processMessage(self, aclRep, payload, curr):
        print 'Processing the message...'
        print 'The message was processed.'
