#!/usr/bin/python
# -*- coding: utf-8 -*-

import xml.dom.minidom
import xml.dom.ext
import Util

class Fuzzy:

    def __init__ (self, a, b, c, d, k = 1.0):

        self.a = float(a)
        self.b = float(b)
        self.c = float(c)
        self.d = float(d)
        self.k = float(k)

    def domain (self):

        a = min(self.a, self.b, self.c, self.d)
        b = max(self.a, self.b, self.c, self.d)
        return (a, b)

    def copy (self):
        return Fuzzy(self.a, self.b, self.c, self.d)

    def eval (self, num):

        if num <= self.a:
            return 0.0
        elif (num > self.a) and (num < self.b) and (self.a <> self.b):
            return ((num - self.a) / (self.b - self.a)) * self.k
        elif (num >= self.b) and (num <= self.c):
            return self.k
        elif (num > self.c) and (num < self.d) and (self.d <> self.c):
            return ((self.d - num) / (self.d - self.c)) * self.k
        else:
            return 0.0

    def __call__ (self, num):
        return self.eval(num)

    def __str__ (self):
        return '[' + str(self.a) + ',' + str(self.b) + ',' + str(self.c) + ',' + str(self.d) + ']'

    def changeK (self, k1):

        self.b = (self.b - self.a) * k1 / self.k + self.a
        self.c = self.d - (self.d - self.c) * k1 / self.k
        self.k = k1

    def area (self):
        return float(((self.d - self.a) + (self.c - self.b)) * self.k / 2.0)

    def cg (self):

        S = self.area()
        if S == 0:
            return self.a
        else:
            # Baricenters.
            b1 = (self.a + self.b * 2.0) / 3.0
            b2 = (self.b + self.c) / 2.0
            b3 = (self.c * 2.0 + self.d) / 3.0
            # Areas.
            a1 = (self.b - self.a) * self.k / 2.0
            a2 = (self.c - self.b) * self.k
            a3 = (self.d - self.c) * self.k / 2.0
            return (a1 * b1 + a2 * b2 + a3 * b3) / S

class FuzzyVar:

    def __init__ (self, name):

        self.name = name
        self.values = {}
        self.domain = None

    def setValue (self, label, value):

        if not self.values.has_key(label):
            self.values[label] = value
            if not (self.domain == None):
                a, b = value.domain()
                self.domain = (min(self.domain[0], a), max(self.domain[1], b))
            else:
                self.domain = value.domain()

    def getValue (self, label, num):

        if not self.values.has_key(label):
            return 0.0
        else:
            return self.values[label](num)

    def __call__ (self, label, num):
        return self.getValue(label, num)

    def allEval (self, num):

        res = {}
        for label in self.values.keys():
            res[label] = self.values[label](num)
        return res

    def allDesf (self):

        res = {}
        for label in self.values.keys():
            res[label] = self.values[label].cg()
        return res

class FuzzySystem:

    def __init__ (self, name = 'NoName', tnorm = min, tconorm = max):

        self.name = name
        self.rules = []
        self.var = {}
        self.resp = None
        self.tnorm = tnorm
        self.tconorm = tconorm

    def insertVar (self, var):
        self.var[var.name] = var

    def insertRule (self, listAnt, listCons):
        self.rules.append((listAnt, listCons))

    def strRule (self, rule):

        ListAnt, ListCons = rule
        straux = 'IF '
        for ant in ListAnt:
            straux = straux + str(ant[0]) + ' IS ' + str(ant[1]) + ' '
        straux = straux + ' THEN '
        for cons in ListCons:
            straux = straux + str(cons[0]) + ' IS ' + str(cons[1]) + ' '
        return straux

    def __str__ (self):

        straux = 'Rule System: ' + self.name + '\n'
        for i in range(0, len(self.rules)):
            straux = straux + '[' + str(i) + ']' + self.strRule(self.rules[i]) + '\n'
        return straux

    def setInput (self, ListInput):

        self.resp = {}
        for input in ListInput:
            var, value = input
            self.resp[var] = self.var[var].allEval(value)
        return self.resp

    def evalAnt (self, rule):

        listant, cons = rule
        seq = []
        for ant in listant:
            name, label = ant
            seq.append(self.resp[name][label])
        return self.tnorm(seq)

    def evalAll (self, ListInput):

        # res represents a list with the tnorms of the antecedents of each rule.
        res = []
        # acu represents the sume of all the elementos of res.
        acu = 0.0
        self.setInput(ListInput)
        # rescons represents the consequents of each rule.
        rescons = {}
        for r in self.rules:
            listAnt, listCons = r
            ant = self.evalAnt(r)
            acu = acu + ant
            res.append(ant)
            if ant == 0.0:
                continue
            for cons in listCons:
                Var, Etiq = cons
                if not rescons.has_key(Var):
                    rescons[Var] = {Etiq:ant}
                elif not rescons[Var].has_key(Etiq):
                    rescons[Var][Etiq] = ant
                else:
                    rescons[Var][Etiq] = self.tconorm(rescons[Var][Etiq], ant)

        return res, acu, rescons

    def out (self, Listinput):

        a, a1, c = self.evalAll(Listinput)
        Resalida = {}
        for var in c.keys():
            sum, acu = 0.0, 0.0
            for etq in c[var]:
                new = self.var[var].values[etq]
                new = new.copy()
                new.changeK(c[var][etq])
                sum = sum + new.cg() * new.area()
                acu = acu + new.area()
                try:
                    res = sum / acu
                except ZeroDivisionError:
                    res = sum
            Resalida[var] = res
        return Resalida

class UtilFuzzy:

    def __init__ (self):

        self.SystemName = ''
        self.Variables = []
        self.Antecedents = []
        self.Consequences = []

    def setSystemName (self, systemName):
        self.SystemName = systemName

    def getSystemName (self):
        return self.SystemName

    def insertVariable (self, var):
        self.Variables.append(var)

    def getVariables (self):
        return self.Variables

    def insertAntecedent (self, antecedent):
        self.Antecedents.append(antecedent)

    def insertConsequence (self, consequence):
        self.Consequences.append(consequence)

    def getAntecedents (self):
        return self.Antecedents

    def getConsequences (self):
        return self.Consequences

    def loadFile (self, file):

        doc = xml.dom.minidom.parse(file)
        system = doc.firstChild.childNodes[1]

        # Establish the system name.
        self.setSystemName(system.getAttribute('name'))

        # Establish the variables and rules of the system.
        for x in system.childNodes:

            # Linguistic variables.
            if x.nodeType == x.ELEMENT_NODE and x.nodeName == 'linguisticvar':
                name, type = x.getAttribute('name'), x.getAttribute('type')
                var = FuzzyVar(name)

                # Obtain the fuzzy sets of this variable.
                for y in x.childNodes:
                    if y.nodeType == y.ELEMENT_NODE and y.nodeName == 'fuzzyset':
                        label = y.getAttribute('label')
                        a, b, c, d = y.getAttribute('a'), y.getAttribute('b'), y.getAttribute('c'), y.getAttribute('d')
                        var.setValue(label, Fuzzy(a, b, c, d))

                self.insertVariable(var)

                var = None

            # Rule.
            elif x.nodeType == x.ELEMENT_NODE and x.nodeName == 'rule':
                ant, cons = [], []
                
                for y in x.childNodes:
                    if y.nodeType == y.ELEMENT_NODE and y.nodeName == 'antecedent':
                        ant.append((y.getAttribute('varname'), y.getAttribute('linguisticlabel')))
                    elif y.nodeType == y.ELEMENT_NODE and y.nodeName == 'consequence':
                        cons.append((y.getAttribute('varname'), y.getAttribute('linguisticlabel')))

                # Insert the antecedents and consequents of the rule i.
                self.insertAntecedent(ant)
                self.insertConsequence(cons)

    def loadInputData (self, s, co, nd, file):

        doc = xml.dom.minidom.parse(file)
        system = doc.firstChild.childNodes[1]

        # Establish the variables and rules of the system.
        for x in system.childNodes:
            
            # Linguistic variable.
            if x.nodeType == x.ELEMENT_NODE and x.nodeName == 'linguisticvar':
                name, type = x.getAttribute('name'), x.getAttribute('type')
                var = FuzzyVar(name)

                # Size.
                if name == 'S':
                    for y in x.childNodes:
                        if y.nodeType == y.ELEMENT_NODE and y.nodeName == 'fuzzyset':
                            label = y.getAttribute('label')
                            a, b, c, d = s[label][0], s[label][1], s[label][2], s[label][3]
                            y.setAttribute('a', a)
                            y.setAttribute('b', b)
                            y.setAttribute('c', c)
                            y.setAttribute('d', d)

                # Complexity.
                elif name == 'C':
                    for y in x.childNodes:
                        if y.nodeType == y.ELEMENT_NODE and y.nodeName == 'fuzzyset':
                            label = y.getAttribute('label')
                            a, b, c, d = co[label][0], co[label][1], co[label][2], co[label][3]
                            y.setAttribute('a', a)
                            y.setAttribute('b', b)
                            y.setAttribute('c', c)
                            y.setAttribute('d', d)

                # Neighbour difference.
                elif name == 'Nd':
                    for y in x.childNodes:
                        if y.nodeType == y.ELEMENT_NODE and y.nodeName == 'fuzzyset':
                            label = y.getAttribute('label')
                            a, b, c, d = nd[label][0], nd[label][1], nd[label][2], nd[label][3]
                            y.setAttribute('a', a)
                            y.setAttribute('b', b)
                            y.setAttribute('c', c)
                            y.setAttribute('d', d)

        f = open(file, 'w')
        xml.dom.ext.PrettyPrint(doc, f)
        f.close()

if __name__ == '__main__':

    fuzzy = UtilFuzzy()
    fuzzy.loadFile('./prueba.xml')
    s, c, nd = Util.loadInput('./prueba.txt')
    fuzzy.loadInputData(s, c, nd, './prueba.xml')

    InputData = []
    InputData.append((('C', 8), ('S', 38400), ('Op', 15.0), ('Nd', 33)))
    
    System = FuzzySystem(fuzzy.getSystemName())
    
    for x in fuzzy.getVariables():
        System.insertVar(x)

    print System.var.keys()
    print ''
    for x in System.var.values():
        print x.values.keys()
        for y in x.values.values():
            print y.a, y.b, y.c, y.d
        print ''

    for i in range(len(fuzzy.getAntecedents())):
        System.insertRule(fuzzy.getAntecedents()[i], fuzzy.getConsequences()[i])

    for x in InputData:
        a, b, c = System.evalAll(x)
        print a
        print b
        print c
        print System.out(x)
